# ruff: noqa: N802

from enum import StrEnum

from ._utils import (
    CaseInsensitiveIndexedBaseModel,
    class_cached_property,
)
from .factions import Faction


class Orientation(StrEnum):
    HORIZONTAL = "horizontal"
    VERTICAL = "vertical"


class Map(CaseInsensitiveIndexedBaseModel):
    name: str
    tag: str
    pretty_name: str
    short_name: str
    allies: Faction
    axis: Faction
    orientation: Orientation
    """Whether the sectors on this map are stacked horizontally (left-to-right) or
    vertically (top-to-bottom)."""
    mirror_factions: bool
    """If the side each faction starts at is mirrored or not. By default, Allies spawn
    left/top, Axis spawn right/bottom."""

    def __str__(self) -> str:
        return self.id

    def __repr__(self) -> str:
        return (
            f"{self.__class__.__name__}(id={self.id!r}, name={self.name!r}, "
            f"tag={self.tag!r}, pretty_name={self.pretty_name!r}, "
            f"short_name={self.short_name!r}, allies={self.allies!r}, "
            f"axis={self.axis!r}, orientation={self.orientation!r}, "
            f"mirror_factions={self.mirror_factions!r})"
        )

    def __hash__(self) -> int:
        return hash(self.id)

    def __eq__(self, other: object) -> bool:
        if isinstance(other, (Map, str)):
            return str(self).lower() == str(other).lower()
        return NotImplemented

    @class_cached_property
    @classmethod
    def CARENTAN(cls) -> "Map":
        return cls(
            id="carentan",
            name="CARENTAN",
            tag="CAR",
            pretty_name="Carentan",
            short_name="Carentan",
            allies=Faction.US,
            axis=Faction.GER,
            orientation=Orientation.HORIZONTAL,
            mirror_factions=False,
        )

    @class_cached_property
    @classmethod
    def DRIEL(cls) -> "Map":
        return cls(
            id="driel",
            name="DRIEL",
            tag="DRL",
            pretty_name="Driel",
            short_name="Driel",
            allies=Faction.CW,
            axis=Faction.GER,
            orientation=Orientation.VERTICAL,
            mirror_factions=True,
        )

    @class_cached_property
    @classmethod
    def EL_ALAMEIN(cls) -> "Map":
        return cls(
            id="elalamein",
            name="EL ALAMEIN",
            tag="ELA",
            pretty_name="El Alamein",
            short_name="Alamein",
            allies=Faction.B8A,
            axis=Faction.DAK,
            orientation=Orientation.HORIZONTAL,
            mirror_factions=True,
        )

    @class_cached_property
    @classmethod
    def ELSENBORN_RIDGE(cls) -> "Map":
        return cls(
            id="elsenbornridge",
            name="ELSENBORN RIDGE",
            tag="EBR",
            pretty_name="Elsenborn Ridge",
            short_name="Elsenborn",
            allies=Faction.US,
            axis=Faction.GER,
            orientation=Orientation.VERTICAL,
            mirror_factions=False,
        )

    @class_cached_property
    @classmethod
    def FOY(cls) -> "Map":
        return cls(
            id="foy",
            name="FOY",
            tag="FOY",
            pretty_name="Foy",
            short_name="Foy",
            allies=Faction.US,
            axis=Faction.GER,
            orientation=Orientation.VERTICAL,
            mirror_factions=True,
        )

    @class_cached_property
    @classmethod
    def HILL_400(cls) -> "Map":
        return cls(
            id="hill400",
            name="HILL 400",
            tag="HIL",
            pretty_name="Hill 400",
            short_name="Hill 400",
            allies=Faction.US,
            axis=Faction.GER,
            orientation=Orientation.HORIZONTAL,
            mirror_factions=False,
        )

    @class_cached_property
    @classmethod
    def HURTGEN_FOREST(cls) -> "Map":
        return cls(
            id="hurtgenforest",
            name="HÜRTGEN FOREST",
            tag="HUR",
            pretty_name="Hurtgen Forest",
            short_name="Hurtgen",
            allies=Faction.US,
            axis=Faction.GER,
            orientation=Orientation.HORIZONTAL,
            mirror_factions=False,
        )

    @class_cached_property
    @classmethod
    def KHARKOV(cls) -> "Map":
        return cls(
            id="kharkov",
            name="Kharkov",
            tag="KHA",
            pretty_name="Kharkov",
            short_name="Kharkov",
            allies=Faction.SOV,
            axis=Faction.GER,
            orientation=Orientation.VERTICAL,
            mirror_factions=False,
        )

    @class_cached_property
    @classmethod
    def KURSK(cls) -> "Map":
        return cls(
            id="kursk",
            name="KURSK",
            tag="KUR",
            pretty_name="Kursk",
            short_name="Kursk",
            allies=Faction.SOV,
            axis=Faction.GER,
            orientation=Orientation.VERTICAL,
            mirror_factions=False,
        )

    @class_cached_property
    @classmethod
    def MORTAIN(cls) -> "Map":
        return cls(
            id="mortain",
            name="MORTAIN",
            tag="MOR",
            pretty_name="Mortain",
            short_name="Mortain",
            allies=Faction.US,
            axis=Faction.GER,
            orientation=Orientation.HORIZONTAL,
            mirror_factions=False,
        )

    @class_cached_property
    @classmethod
    def OMAHA_BEACH(cls) -> "Map":
        return cls(
            id="omahabeach",
            name="OMAHA BEACH",
            tag="OMA",
            pretty_name="Omaha Beach",
            short_name="Omaha",
            allies=Faction.US,
            axis=Faction.GER,
            orientation=Orientation.HORIZONTAL,
            mirror_factions=True,
        )

    @class_cached_property
    @classmethod
    def PURPLE_HEART_LANE(cls) -> "Map":
        return cls(
            id="purpleheartlane",
            name="PURPLE HEART LANE",
            tag="PHL",
            pretty_name="Purple Heart Lane",
            short_name="PHL",
            allies=Faction.US,
            axis=Faction.GER,
            orientation=Orientation.VERTICAL,
            mirror_factions=False,
        )

    @class_cached_property
    @classmethod
    def REMAGEN(cls) -> "Map":
        return cls(
            id="remagen",
            name="REMAGEN",
            tag="REM",
            pretty_name="Remagen",
            short_name="Remagen",
            allies=Faction.US,
            axis=Faction.GER,
            orientation=Orientation.VERTICAL,
            mirror_factions=True,
        )

    @class_cached_property
    @classmethod
    def ST_MARIE_DU_MONT(cls) -> "Map":
        return cls(
            id="stmariedumont",
            name="ST MARIE DU MONT",
            tag="SMDM",
            pretty_name="St. Marie Du Mont",
            short_name="SMDM",
            allies=Faction.US,
            axis=Faction.GER,
            orientation=Orientation.VERTICAL,
            mirror_factions=False,
        )

    @class_cached_property
    @classmethod
    def ST_MERE_EGLISE(cls) -> "Map":
        return cls(
            id="stmereeglise",
            name="SAINTE-MÈRE-ÉGLISE",
            tag="SME",
            pretty_name="St. Mere Eglise",
            short_name="SME",
            allies=Faction.US,
            axis=Faction.GER,
            orientation=Orientation.HORIZONTAL,
            mirror_factions=True,
        )

    @class_cached_property
    @classmethod
    def STALINGRAD(cls) -> "Map":
        return cls(
            id="stalingrad",
            name="STALINGRAD",
            tag="STA",
            pretty_name="Stalingrad",
            short_name="Stalingrad",
            allies=Faction.SOV,
            axis=Faction.GER,
            orientation=Orientation.HORIZONTAL,
            mirror_factions=True,
        )

    @class_cached_property
    @classmethod
    def TOBRUK(cls) -> "Map":
        return cls(
            id="tobruk",
            name="TOBRUK",
            tag="TBK",
            pretty_name="Tobruk",
            short_name="Tobruk",
            allies=Faction.B8A,
            axis=Faction.DAK,
            orientation=Orientation.HORIZONTAL,
            mirror_factions=True,
        )

    @class_cached_property
    @classmethod
    def UTAH_BEACH(cls) -> "Map":
        return cls(
            id="utahbeach",
            name="UTAH BEACH",
            tag="UTA",
            pretty_name="Utah Beach",
            short_name="Utah",
            allies=Faction.US,
            axis=Faction.GER,
            orientation=Orientation.HORIZONTAL,
            mirror_factions=True,
        )



# ruff: noqa: T201

import json
from pathlib import Path
from typing import NamedTuple


class Strongpoint(NamedTuple):
    center: tuple[float, float, float]
    radius: float


def sort_key_horizontal(sp: Strongpoint) -> float:
    return (round(sp.center[0] / 40000) * 40000) + (sp.center[1] / 40000)


def sort_key_vertical(sp: Strongpoint) -> float:
    return (round(sp.center[1] / 40000) * 40000) + (sp.center[0] / 40000)


def main() -> None:
    content = json.loads(Path("scripts/extract_strongpoints.json").read_text())
    strongpoints: list[Strongpoint] = []
    for actor in content:
        if actor["Name"] == "TriggerShape" and actor["Outer"].startswith(
            "SphereSectorCaptureBooster",
        ):
            props = actor["Properties"]
            center = props["RelativeLocation"]
            radius = props.get("SphereRadius", 1000) * props.get(
                "RelativeScale3D",
                {},
            ).get("X", 1.0)

            strongpoints.append(
                Strongpoint(
                    center=(center["X"], center["Y"], center["Z"]),
                    radius=radius,
                ),
            )

    if len(strongpoints) != 15:
        msg = f"Expected 15 strongpoints, got {len(strongpoints)}"
        raise ValueError(msg)

    all_x = [sp.center[0] for sp in strongpoints]
    dx = max(all_x) - min(all_x)
    if dx > 120000:
        strongpoints.sort(key=sort_key_horizontal)
    else:
        strongpoints.sort(key=sort_key_vertical)

    for strongpoint in strongpoints:
        print(f", center={strongpoint.center}, radius={strongpoint.radius}")


if __name__ == "__main__":
    main()



from abc import ABC, abstractmethod
from collections.abc import AsyncGenerator
from contextlib import asynccontextmanager

from hllrcon.commands import RconCommands


class RconClient(RconCommands, ABC):
    """Abstract base class for RCON clients.

    This class defines the interface for RCON clients, including methods for
    connecting, disconnecting, sending commands.
    """

    @abstractmethod
    def is_connected(self) -> bool:
        """Check if the client is connected to the RCON server.

        Returns
        -------
        bool
            True if connected, False otherwise.

        """

    @abstractmethod
    @asynccontextmanager
    async def connect(self) -> AsyncGenerator[None]:
        """Establish a connection to the RCON server."""
        yield

    @abstractmethod
    async def wait_until_connected(self) -> None:
        """Wait until the client is connected to the RCON server.

        This might be useful to verify that a connection can be established before
        continuing with other operations.
        """

    @abstractmethod
    def disconnect(self) -> None:
        """Disconnect from the RCON server."""



# ruff: noqa: N802

from typing import TYPE_CHECKING, ClassVar

from ._utils import IndexedBaseModel, class_cached_property
from .teams import Team


class Faction(IndexedBaseModel[int, None]):
    UNASSIGNED_ID: ClassVar[int] = 6

    id: int
    name: str
    short_name: str
    team: Team

    @classmethod
    def _lookup_fallback(cls, id_: int) -> "Faction | None":
        if id_ == cls.UNASSIGNED_ID:
            return None

        return super()._lookup_fallback(id_)

    if TYPE_CHECKING:

        @classmethod
        def by_id(cls, id_: int) -> "Faction | None":
            """Look up a faction by its identifier.

            Parameters
            ----------
            id_ : int
                The identifier of the faction to look up.

            Returns
            -------
            Faction | None
                The faction instance with the given identifier, or `None` if the
                identifier corresponds with being unassigned.

            Raises
            ------
            ValueError
                If no faction with the given identifier exists.

            """

    @property
    def is_allied(self) -> bool:
        """Whether the faction is part of the allied forces.

        Allied factions are:
        - United States (`US`)
        - Soviet Union (`SOV`)
        - Commonwealth (`CW`)
        - British 8th Army (`B8A`)
        """
        return self.team == Team.ALLIES

    @property
    def is_axis(self) -> bool:
        """Whether the faction is part of the axis forces.

        Axis factions are:
        - Germany (`GER`)
        - German Africa Corps (`DAK`)
        """
        return self.team == Team.AXIS

    @class_cached_property
    @classmethod
    def GER(cls) -> "Faction":
        return cls(
            id=0,
            name="Germany",
            short_name="GER",
            team=Team.AXIS,
        )

    @class_cached_property
    @classmethod
    def US(cls) -> "Faction":
        return cls(
            id=1,
            name="United States",
            short_name="US",
            team=Team.ALLIES,
        )

    @class_cached_property
    @classmethod
    def SOV(cls) -> "Faction":
        return cls(
            id=2,
            name="Soviet Union",
            short_name="SOV",
            team=Team.ALLIES,
        )

    @class_cached_property
    @classmethod
    def RUS(cls) -> "Faction":
        return cls.SOV

    @class_cached_property
    @classmethod
    def CW(cls) -> "Faction":
        return cls(
            id=3,
            name="Allies",
            short_name="CW",
            team=Team.ALLIES,
        )

    @class_cached_property
    @classmethod
    def GB(cls) -> "Faction":
        return cls.CW

    @class_cached_property
    @classmethod
    def DAK(cls) -> "Faction":
        return cls(
            id=4,
            name="German Africa Corps",
            short_name="DAK",
            team=Team.AXIS,
        )

    @class_cached_property
    @classmethod
    def B8A(cls) -> "Faction":
        return cls(
            id=5,
            name="British Eighth Army",
            short_name="B8A",
            team=Team.ALLIES,
        )



import asyncio
import contextlib
from typing import Any
from unittest import mock

import pytest
from hllrcon.exceptions import HLLError
from hllrcon.pooled.rcon import PooledRcon
from hllrcon.pooled.worker import PooledRconWorker
from pytest_mock import MockerFixture

pytestmark = pytest.mark.filterwarnings("ignore::hllrcon.exceptions.HLLRconWarning")


@pytest.fixture
def pool(mocker: MockerFixture) -> PooledRcon:
    def worker_factory(*_args: Any, **_kwargs: dict[str, Any]) -> mock.Mock:  # noqa: ANN401
        mock_worker = mock.Mock(PooledRconWorker)
        mock_worker.is_connected.return_value = True
        mock_worker.is_disconnected.return_value = False
        return mock_worker

    mocker.patch(
        "hllrcon.pooled.rcon.PooledRconWorker",
        side_effect=worker_factory,
    )

    return PooledRcon(
        host="localhost",
        port=1234,
        password="password",
        max_workers=2,
    )


@pytest.mark.asyncio
async def test_get_worker_empty_pool(pool: PooledRcon) -> None:
    async with pool._get_available_worker():
        assert len(pool._workers) == 1
        assert pool._queue.empty()

    assert len(pool._workers) == 1
    assert pool._queue.qsize() == 1


@pytest.mark.asyncio
async def test_get_worker_one_available(pool: PooledRcon) -> None:
    async with pool._get_available_worker() as worker1:
        pass

    async with pool._get_available_worker() as worker2:
        assert worker1 is worker2
        assert len(pool._workers) == 1
        assert pool._queue.qsize() == 0

    assert len(pool._workers) == 1
    assert pool._queue.qsize() == 1


@pytest.mark.asyncio
async def test_get_worker_add_new(pool: PooledRcon) -> None:
    async with (
        pool._get_available_worker() as worker1,
        pool._get_available_worker() as worker2,
    ):
        assert worker1 is not worker2
        assert len(pool._workers) == 2
        assert pool._queue.qsize() == 0

    assert len(pool._workers) == 2
    assert pool._queue.qsize() == 2


@pytest.mark.asyncio
async def test_get_worker_wait(pool: PooledRcon) -> None:
    async with pool._get_available_worker():
        async with pool._get_available_worker() as worker2:
            with pytest.raises(asyncio.TimeoutError):
                async with asyncio.timeout(0.1):
                    async with pool._get_available_worker():
                        pass

        async with asyncio.timeout(0.1):
            async with pool._get_available_worker() as worker3:
                assert worker2 is worker3


@pytest.mark.asyncio
async def test_get_worker_replace_disconnected(pool: PooledRcon) -> None:
    async with pool._get_available_worker():
        async with pool._get_available_worker() as worker2:
            pass

        worker2.is_disconnected.return_value = True  # type: ignore[attr-defined]
        pool._workers.remove(worker2)

        async with asyncio.timeout(0.1):
            async with pool._get_available_worker() as worker3:
                assert worker2 is not worker3


@pytest.mark.parametrize("max_workers", [0, -1])
def test_max_workers_too_small(max_workers: int) -> None:
    with pytest.raises(ValueError, match="Max workers must be greater than 0"):
        PooledRcon(
            host="localhost",
            port=1234,
            password="password",
            max_workers=max_workers,
        )


@pytest.mark.asyncio
async def test_is_connected(pool: PooledRcon) -> None:
    assert pool.is_connected() is False, "Pool should not be connected initially"

    async with pool._get_available_worker():
        assert pool.is_connected() is True

    assert pool.is_connected() is True

    pool.disconnect()
    assert pool.is_connected() is False, "Pool should not be connected after disconnect"


@pytest.mark.asyncio
async def test_num_workers(pool: PooledRcon) -> None:
    assert pool.num_workers == 0, "Pool should have no workers initially"

    async with pool._get_available_worker():
        assert pool.num_workers == 1

        async with pool._get_available_worker():
            assert pool.num_workers == 2

    assert pool.num_workers == 2

    pool.disconnect()
    assert pool.num_workers == 0


@pytest.mark.asyncio
async def test_execute(pool: PooledRcon) -> None:
    command = "command"
    version = 1
    body = "body"

    async with pool._get_available_worker() as worker:
        worker.execute = mock.AsyncMock(return_value="result")  # type: ignore[method-assign]

    result = await pool.execute(command, version, body)

    worker.execute.assert_called_once_with(command, version, body)
    assert result == "result"


@pytest.mark.asyncio
async def test_enter_exit(pool: PooledRcon) -> None:
    async with pool.connect(), pool._get_available_worker() as worker:
        assert pool._workers == [worker], "Worker should be added to the pool"

    assert not pool._workers, "Workers should be cleared after exit"
    assert pool._queue.empty(), "Queue should be empty after exit"

    with contextlib.suppress(RuntimeError):
        async with pool.connect(), pool._get_available_worker() as worker:
            assert pool._workers == [worker], "Worker should be added to the pool again"

            # Raise error this time
            raise RuntimeError

    assert not pool._workers, "Workers should be cleared after error"
    assert pool._queue.empty(), "Queue should be empty after error"


@pytest.mark.asyncio
async def test_wait_until_connected(pool: PooledRcon) -> None:
    assert not pool.is_connected(), "Pool should not be connected initially"

    await pool.wait_until_connected()
    assert pool.is_connected(), "Pool should be connected after waiting"
    assert pool.num_workers == 1, "Pool should have one worker after waiting"

    # Test connected worker
    await pool.wait_until_connected()

    worker = pool._workers[0]

    # Test connecting worker
    worker.is_connected.return_value = False  # type: ignore[attr-defined]
    worker.wait_until_connected = mock.AsyncMock()  # type: ignore[method-assign]
    await pool.wait_until_connected()
    worker.wait_until_connected.assert_awaited_once()

    # Test disconnected worker
    worker.is_disconnected.return_value = True  # type: ignore[attr-defined]
    worker.wait_until_connected.side_effect = HLLError
    await pool.wait_until_connected()



import itertools
import json
import struct
from typing import Any, ClassVar

from hllrcon.protocol.constants import REQUEST_HEADER_FORMAT


class RconRequest:
    """Represents a RCON request."""

    __request_id_counter: ClassVar["itertools.count[int]"] = itertools.count(start=0)

    def __init__(
        self,
        command: str,
        version: int,
        auth_token: str | None,
        content_body: dict[str, Any] | str = "",
    ) -> None:
        """Initializes a new RCON request.

        Parameters
        ----------
        command : str
            The command to be executed.
        version : int
            The version of the command.
        auth_token : str | None
            The authentication token for the RCON connection.
        content_body : dict[str, Any] | str, optional
            An additional payload to send along with the command. Must be
            JSON-serializable.

        """
        self.name = command
        self.version = version
        self.auth_token = auth_token
        self.content_body = content_body
        self.request_id: int = next(self.__request_id_counter)

    def pack(self) -> tuple[bytes, bytes]:
        """Packs the request into a bytes object.

        Returns
        -------
        tuple[bytes, bytes]
            A tuple containing the header and body of the request.

        """
        body = {
            "authToken": self.auth_token or "",
            "version": self.version,
            "name": self.name,
            "contentBody": (
                self.content_body
                if isinstance(self.content_body, str)
                else json.dumps(self.content_body, separators=(",", ":"))
            ),
        }
        body_encoded = json.dumps(body, separators=(",", ":")).encode()
        header = struct.pack(
            REQUEST_HEADER_FORMAT,
            self.request_id,
            len(body_encoded),
        )
        return header, body_encoded



import asyncio
import contextlib
import logging
from typing import Any
from unittest import mock

import pytest
from hllrcon.connection import RconConnection
from hllrcon.exceptions import HLLError
from hllrcon.rcon import Rcon

pytestmark = pytest.mark.asyncio


@pytest.fixture
def connection() -> RconConnection:
    mock_connection = mock.Mock(spec=RconConnection)
    mock_connection.is_connected.return_value = True
    return mock_connection


@pytest.fixture
def connection2() -> RconConnection:
    mock_connection = mock.Mock(spec=RconConnection)
    mock_connection.is_connected.return_value = True
    return mock_connection


@pytest.fixture
def rcon(monkeypatch: pytest.MonkeyPatch, connection: mock.Mock) -> Rcon:
    async def get_connection(*_args: Any, **_kwargs: dict[str, Any]) -> RconConnection:  # noqa: ANN401
        return connection

    monkeypatch.setattr("hllrcon.rcon.RconConnection.connect", get_connection)
    return Rcon(host="localhost", port=1234, password="password")


async def test_logger(rcon: Rcon) -> None:
    assert rcon.logger.name == "hllrcon.rcon"
    rcon.logger = logging.getLogger("test")
    assert rcon.logger.name == "test"
    rcon.logger = None
    assert rcon.logger.name == "hllrcon.rcon"


async def test_get_connection_new(rcon: Rcon, connection: mock.Mock) -> None:
    assert await rcon._get_connection() == connection


async def test_get_connection_reuse(
    rcon: Rcon,
    connection2: mock.Mock,
) -> None:
    rcon._connection = asyncio.Future()
    rcon._connection.set_result(connection2)
    assert await rcon._get_connection() == connection2


async def test_get_connection_disconnected(
    rcon: Rcon,
    connection: mock.Mock,
    connection2: mock.Mock,
) -> None:
    rcon._connection = asyncio.Future()
    rcon._connection.set_result(connection2)
    connection2.is_connected.return_value = False

    assert await rcon._get_connection() == connection


async def test_get_connection_wait(
    rcon: Rcon,
    connection2: mock.Mock,
) -> None:
    rcon._connection = asyncio.Future()

    with pytest.raises(asyncio.TimeoutError):
        async with asyncio.timeout(0.1):
            await rcon._get_connection()

    rcon._connection.set_result(connection2)
    assert await rcon._get_connection() == connection2


async def test_get_connection_failure(
    monkeypatch: pytest.MonkeyPatch,
    rcon: Rcon,
    connection: mock.Mock,
) -> None:
    with monkeypatch.context() as m:

        async def get_connection(
            *_args: Any,  # noqa: ANN401
            **_kwargs: dict[str, Any],
        ) -> RconConnection:
            msg = "Connection failed"
            raise HLLError(msg)

        m.setattr("hllrcon.rcon.RconConnection.connect", get_connection)

        with pytest.raises(HLLError, match="Connection failed"):
            await rcon._get_connection()

    assert await rcon._get_connection() == connection


async def test_is_connected(rcon: Rcon, connection: mock.Mock) -> None:
    assert rcon.is_connected() is False, "Should be disconnected initially"

    await rcon.wait_until_connected()
    assert rcon.is_connected() is True, "Should be connected after getting connection"

    connection.is_connected.return_value = False
    assert rcon.is_connected() is False, "Should be disconnected after connection loss"

    rcon._connection = asyncio.Future()
    rcon._connection.set_exception(HLLError("Connection lost"))
    assert rcon.is_connected() is False, "Should be disconnected after exception"

    rcon._connection = asyncio.Future()
    rcon._connection.cancel()
    assert rcon.is_connected() is False, "Should be disconnected after cancelling"


async def test_enter_exit(rcon: Rcon, connection: mock.Mock) -> None:
    assert rcon._connection is None, "Initial connection should be None"

    async with rcon.connect():
        assert rcon._connection is not None, "Connection should be established"

    connection.disconnect.assert_called_once()
    assert rcon._connection is None, "Connection should be reset to None after exit"

    connection.disconnect.reset_mock()

    with contextlib.suppress(RuntimeError):
        async with rcon.connect():
            assert rcon._connection is not None, (
                "Connection should be established again"
            )

            # Raise error this time
            raise RuntimeError

    connection.disconnect.assert_called_once()
    assert rcon._connection is None, "Connection should be reset to None after error"


async def test_aexit_no_connection(rcon: Rcon) -> None:
    async with rcon.connect():
        rcon._connection = None

    assert rcon._connection is None


async def test_aexit_reconnecting(rcon: Rcon) -> None:
    fut: asyncio.Future[RconConnection] = asyncio.Future()

    async with rcon.connect():
        rcon._connection = fut

    assert rcon._connection is None
    assert fut.cancelled() is True


async def test_aexit_connection_failure(rcon: Rcon) -> None:
    fut: asyncio.Future[RconConnection] = asyncio.Future()
    fut.set_exception(HLLError("Connection failed"))

    async with rcon.connect():
        rcon._connection = fut

    assert rcon._connection is None


async def test_execute(
    rcon: Rcon,
    connection: mock.Mock,
) -> None:
    command = "command"
    version = 2
    body = "body"
    response = "response"

    connection.execute.return_value = response
    result = await rcon.execute(command, version, body)

    assert result == response


async def test_reconnect_after_failures_parameter() -> None:
    # Test default value
    rcon = Rcon(host="localhost", port=1234, password="password")
    assert rcon.reconnect_after_failures == 3

    # Test custom value
    rcon = Rcon(
        host="localhost",
        port=1234,
        password="password",
        reconnect_after_failures=5,
    )
    assert rcon.reconnect_after_failures == 5

    # Test zero value (disabled)
    rcon = Rcon(
        host="localhost",
        port=1234,
        password="password",
        reconnect_after_failures=0,
    )
    assert rcon.reconnect_after_failures == 0

    # Test negative value gets clamped to zero
    rcon = Rcon(
        host="localhost",
        port=1234,
        password="password",
        reconnect_after_failures=-1,
    )
    assert rcon.reconnect_after_failures == 0


async def test_failure_count_property() -> None:
    rcon = Rcon(host="localhost", port=1234, password="password")

    # Initial failure count should be 0
    assert rcon._failure_count == 0


async def test_failure_count_increment_on_timeout(
    rcon: Rcon,
    connection: mock.Mock,
) -> None:
    # Mock connection.execute to raise TimeoutError
    connection.execute.side_effect = TimeoutError("Connection timeout")

    # Execute command and expect TimeoutError to be raised
    with pytest.raises(TimeoutError, match="Connection timeout"):
        await rcon.execute("test_command", 1, "")

    # Failure count should be incremented
    assert rcon._failure_count == 1

    # Execute another command that times out
    with pytest.raises(TimeoutError, match="Connection timeout"):
        await rcon.execute("test_command", 1, "")

    # Failure count should be incremented again
    assert rcon._failure_count == 2


async def test_failure_count_increment_on_os_error(
    rcon: Rcon,
    connection: mock.Mock,
) -> None:
    # Mock connection.execute to raise OSError
    connection.execute.side_effect = OSError("Network error")

    # Execute command and expect OSError to be raised
    with pytest.raises(OSError, match="Network error"):
        await rcon.execute("test_command", 1, "")

    # Failure count should be incremented
    assert rcon._failure_count == 1


async def test_failure_count_reset_on_disconnect(rcon: Rcon) -> None:
    # Set failure count manually
    rcon._failure_count = 5

    # Disconnect should reset failure count
    rcon.disconnect()
    assert rcon._failure_count == 0


async def test_reconnect_after_failures_disabled(
    monkeypatch: pytest.MonkeyPatch,
    connection: mock.Mock,
) -> None:
    # Create rcon with reconnect_after_failures disabled (0)
    async def get_connection(*_args: Any, **_kwargs: dict[str, Any]) -> RconConnection:  # noqa: ANN401
        return connection

    monkeypatch.setattr("hllrcon.rcon.RconConnection.connect", get_connection)
    rcon = Rcon(
        host="localhost",
        port=1234,
        password="password",
        reconnect_after_failures=0,
    )

    # Mock connection.execute to raise TimeoutError
    connection.execute.side_effect = TimeoutError("Connection timeout")

    # Execute multiple commands that timeout
    for i in range(10):  # Try many times to ensure reconnect doesn't happen
        with pytest.raises(TimeoutError, match="Connection timeout"):
            await rcon.execute("test_command", 1, "")

        # Failure count should keep incrementing
        assert rcon._failure_count == i + 1
        # Connection should still be active (not None)
        assert rcon._connection is not None


async def test_reconnect_after_failures_triggers_disconnect(
    monkeypatch: pytest.MonkeyPatch,
    connection: mock.Mock,
) -> None:
    # Create rcon with reconnect_after_failures = 2
    async def get_connection(*_args: Any, **_kwargs: dict[str, Any]) -> RconConnection:  # noqa: ANN401
        return connection

    monkeypatch.setattr("hllrcon.rcon.RconConnection.connect", get_connection)
    rcon = Rcon(
        host="localhost",
        port=1234,
        password="password",
        reconnect_after_failures=2,
    )

    # Ensure connection is established
    await rcon.wait_until_connected()
    assert rcon._connection is not None

    # Mock connection.execute to raise TimeoutError
    connection.execute.side_effect = TimeoutError("Connection timeout")

    # First failure
    with pytest.raises(TimeoutError, match="Connection timeout"):
        await rcon.execute("test_command", 1, "")
    assert rcon._failure_count == 1
    assert rcon._connection is not None  # Should still be connected

    # Second failure - should trigger disconnect
    with pytest.raises(TimeoutError, match="Connection timeout"):
        await rcon.execute("test_command", 1, "")
    assert rcon._failure_count == 0  # Reset to 0 after disconnect
    assert rcon._connection is None  # Should be disconnected


async def test_failure_count_reset_on_successful_response(
    rcon: Rcon,
    connection: mock.Mock,
) -> None:
    # Set failure count manually
    rcon._failure_count = 5

    # Mock successful response
    connection.execute.return_value = "success"

    # Execute command successfully
    result = await rcon.execute("test_command", 1, "")
    assert result == "success"

    # Failure count should remain unchanged (not reset on success)
    # This is based on the code - it only resets on disconnect
    assert rcon._failure_count == 5


async def test_failure_count_with_different_exception_types(
    rcon: Rcon,
    connection: mock.Mock,
) -> None:
    # Test that only TimeoutError and OSError increment failure count

    # First, test with an exception that should NOT increment failure count
    connection.execute.side_effect = ValueError("Some other error")

    with pytest.raises(ValueError, match="Some other error"):
        await rcon.execute("test_command", 1, "")

    # Failure count should remain 0
    assert rcon._failure_count == 0

    # Now test with TimeoutError
    connection.execute.side_effect = TimeoutError("Timeout")

    with pytest.raises(TimeoutError, match="Timeout"):
        await rcon.execute("test_command", 1, "")

    # Failure count should be incremented
    assert rcon._failure_count == 1

    # Test with OSError
    connection.execute.side_effect = OSError("OS Error")

    with pytest.raises(OSError, match="OS Error"):
        await rcon.execute("test_command", 1, "")

    # Failure count should be incremented again
    assert rcon._failure_count == 2


async def test_reconnect_threshold_exact_match(
    monkeypatch: pytest.MonkeyPatch,
    connection: mock.Mock,
) -> None:
    # Test that disconnect happens exactly when failure count equals threshold
    async def get_connection(*_args: Any, **_kwargs: dict[str, Any]) -> RconConnection:  # noqa: ANN401
        return connection

    monkeypatch.setattr("hllrcon.rcon.RconConnection.connect", get_connection)
    rcon = Rcon(
        host="localhost",
        port=1234,
        password="password",
        reconnect_after_failures=3,
    )

    # Ensure connection is established
    await rcon.wait_until_connected()
    connection.execute.side_effect = TimeoutError("Connection timeout")

    # First two failures should not trigger disconnect
    for i in range(2):
        with pytest.raises(TimeoutError, match="Connection timeout"):
            await rcon.execute("test_command", 1, "")
        assert rcon._failure_count == i + 1
        assert rcon._connection is not None

    # Third failure should trigger disconnect
    with pytest.raises(TimeoutError, match="Connection timeout"):
        await rcon.execute("test_command", 1, "")
    assert rcon._failure_count == 0  # Reset after disconnect
    assert rcon._connection is None



# ruff: noqa: N802

from enum import StrEnum

from ._utils import (
    CaseInsensitiveIndexedBaseModel,
    class_cached_property,
)


class GameModeScale(StrEnum):
    LARGE = "large"
    SMALL = "small"


class GameMode(CaseInsensitiveIndexedBaseModel):
    scale: GameModeScale

    def is_large(self) -> bool:
        return self.scale == GameModeScale.LARGE

    def is_small(self) -> bool:
        return self.scale == GameModeScale.SMALL

    @class_cached_property
    @classmethod
    def WARFARE(cls) -> "GameMode":
        return cls(id="warfare", scale=GameModeScale.LARGE)

    @class_cached_property
    @classmethod
    def OFFENSIVE(cls) -> "GameMode":
        return cls(id="offensive", scale=GameModeScale.LARGE)

    @class_cached_property
    @classmethod
    def SKIRMISH(cls) -> "GameMode":
        return cls(id="skirmish", scale=GameModeScale.SMALL)



class HLLError(Exception):
    """Base exception for all HLL-related errors."""


class HLLCommandError(HLLError):
    """Raised when the game server returns an error for a request."""

    def __init__(self, status_code: int, *args: object) -> None:
        """Initialize a new `HLLCommandError` instance.

        Parameters
        ----------
        status_code : int
            The status code returned by the server.
        *args : object
            Additional arguments to pass to the base exception.

        """
        self.status_code = status_code
        super().__init__(*args)

    def __str__(self) -> str:
        """Return a string representation of the error."""
        exc_str = super().__str__()
        header = f"({self.status_code})"
        return f"{header} {exc_str}".rstrip()


class HLLMessageError(HLLError):
    """Raised when the game server returns an unexpected value."""


class HLLConnectionError(HLLError):
    """Generic error for connection errors."""


class HLLConnectionRefusedError(HLLConnectionError):
    """Raised when the connection is refused."""


class HLLAuthError(HLLConnectionError):
    """Raised for failed authentication."""


class HLLConnectionLostError(HLLConnectionError):
    """Raised when the connection to the server is lost."""


class HLLRconWarning(UserWarning):
    """Base warning for all warnings emitted by the hllrcon library."""



import asyncio
import contextlib
from collections.abc import AsyncGenerator
from typing import TYPE_CHECKING, Any

from hllrcon.connection import RconConnection

if TYPE_CHECKING:
    from hllrcon.pooled.rcon import PooledRcon


class PooledRconWorker:
    """A worker for executing RCON commands in a pooled RCON client."""

    def __init__(self, host: str, port: int, password: str, pool: "PooledRcon") -> None:
        """Initializes a new pooled RCON worker.

        Parameters
        ----------
        host : str
            The hostname or IP address of the RCON server.
        port : int
            The port number of the RCON server.
        password : str
            The password for the RCON connection.
        pool : PooledRcon
            The pooled RCON client that manages this worker.

        """
        self.host = host
        self.port = port
        self.password = password
        self.pool = pool

        self._connection: asyncio.Future[RconConnection] | None = None
        self._busy = False
        self._disconnected = False

    async def _get_connection(self) -> RconConnection:
        """Get the RCON connection for this worker.

        A new connection will be established the first time this method is called.

        The worker will not make any attempts to reconnect if the connection is lost.
        Instead, a new worker should be created to handle future requests.

        Raises
        ------
        HLLConnectionError
            The address and port could not be resolved.
        HLLConnectionRefusedError
            The server refused the connection.
        HLLAuthError
            The provided password is incorrect.

        Returns
        -------
        RconConnection
            The RCON connection for this worker.

        """
        if self._connection:
            return await asyncio.shield(self._connection)

        self._connection = asyncio.Future()
        try:
            connection = await RconConnection.connect(
                host=self.host,
                port=self.port,
                password=self.password,
            )
            connection.on_disconnect = self.on_disconnect
            self._connection.set_result(connection)
        except Exception as e:
            self._connection.set_exception(e)
            self.on_disconnect()
            raise
        else:
            return connection

    async def wait_until_connected(self) -> None:
        await self._get_connection()

    def disconnect(self) -> None:
        if self._connection:
            if self._connection.done():
                if (
                    not self._connection.cancelled()
                    and not self._connection.exception()
                ):
                    self._connection.result().disconnect()
            else:
                self._connection.cancel()

        self.on_disconnect()

    def is_busy(self) -> bool:
        """Check if the worker is currently busy executing a command.

        Returns
        -------
        bool
            True if the worker is busy, False otherwise.

        """
        return self._busy

    def is_connected(self) -> bool:
        """Check if the worker is connected to the RCON server.

        Returns
        -------
        bool
            True if the worker is connected, False otherwise.

        """
        return (
            self._connection is not None
            and not self._connection.cancelled()
            and self._connection.done()
            and not self._connection.exception()
            and self._connection.result().is_connected()
        )

    def is_disconnected(self) -> bool:
        """Check if the worker is disconnected from the RCON server.

        Returns
        -------
        bool
            True if the worker is disconnected, False otherwise.

        """
        return self._disconnected

    def on_disconnect(self) -> None:
        """Clean up the worker when the connection is lost."""
        self._busy = False
        self._disconnected = True
        with contextlib.suppress(ValueError):
            self.pool._workers.remove(self)  # noqa: SLF001

    @contextlib.asynccontextmanager
    async def connect(self) -> AsyncGenerator[RconConnection, None]:
        self._busy = True
        try:
            connection = await self._get_connection()
            yield connection
        finally:
            self._busy = False

    async def execute(
        self,
        command: str,
        version: int,
        body: str | dict[str, Any] = "",
    ) -> str:
        """Execute a command on the RCON server.

        Parameters
        ----------
        command : str
            The command to execute.
        version : int
            The version of the command to execute.
        body : str | dict[str, Any], optional
            The body of the command, by default an empty string.

        Returns
        -------
        str
            The response from the server.

        """
        async with self.connect() as connection:
            return await connection.execute(command, version, body)



from .factions import Faction
from .game_modes import GameMode, GameModeScale
from .layers import Layer, TimeOfDay, Weather
from .loadouts import Loadout, LoadoutId, LoadoutItem
from .maps import Map, Orientation
from .roles import Role, RoleType
from .sectors import CaptureZone, Grid, Sector, Strongpoint
from .teams import Team
from .vehicles import Vehicle, VehicleSeat, VehicleSeatType, VehicleType
from .weapons import Weapon, WeaponType

__all__ = (
    "CaptureZone",
    "Faction",
    "GameMode",
    "GameModeScale",
    "Grid",
    "Layer",
    "Loadout",
    "LoadoutId",
    "LoadoutItem",
    "Map",
    "Orientation",
    "Role",
    "RoleType",
    "Sector",
    "SectorRow",
    "Sectors",
    "Strongpoint",
    "Team",
    "TimeOfDay",
    "Vehicle",
    "VehicleSeat",
    "VehicleSeatType",
    "VehicleType",
    "Weapon",
    "WeaponType",
    "Weather",
)



import asyncio
import concurrent
import concurrent.futures
import contextlib
import logging
from typing import Any
from unittest import mock

import pytest
from hllrcon.connection import RconConnection
from hllrcon.sync.rcon import SyncRcon


@pytest.fixture
def connection() -> RconConnection:
    mock_connection = mock.Mock(spec=RconConnection)
    mock_connection.is_connected.return_value = True
    return mock_connection


@pytest.fixture
def rcon(monkeypatch: pytest.MonkeyPatch, connection: mock.Mock) -> SyncRcon:
    async def get_connection(*_args: Any, **_kwargs: dict[str, Any]) -> RconConnection:  # noqa: ANN401
        return connection

    monkeypatch.setattr("hllrcon.rcon.RconConnection.connect", get_connection)
    return SyncRcon(host="localhost", port=1234, password="password")


def test_basic_usage(rcon: SyncRcon, connection: mock.Mock) -> None:
    with rcon.connect():
        connection.execute.return_value = "pong"
        result = rcon.execute("ping", 1)
        assert result == "pong"


def test_properties(rcon: SyncRcon) -> None:
    assert rcon.host == "localhost"
    assert rcon.port == 1234
    assert rcon.password == "password"
    assert rcon.logger.name == "hllrcon.sync.rcon"
    assert rcon.reconnect_after_failures == 3

    rcon.host = "new_host"
    rcon.port = 4321
    rcon.password = "new_password"
    rcon.logger = logging.getLogger("test")
    rcon.reconnect_after_failures = 5

    assert rcon.host == "new_host"
    assert rcon.port == 4321
    assert rcon.password == "new_password"
    assert rcon.logger.name == "test"
    assert rcon.reconnect_after_failures == 5

    assert rcon._rcon.host == "new_host"
    assert rcon._rcon.port == 4321
    assert rcon._rcon.password == "new_password"
    assert rcon._rcon.logger.name == "test"
    assert rcon._rcon.reconnect_after_failures == 5


def test_is_connected(rcon: SyncRcon, connection: mock.Mock) -> None:
    assert rcon.is_connected() is False, "Should be disconnected initially"

    rcon.wait_until_connected()
    assert rcon.is_connected() is True, "Should be connected after getting connection"

    connection.is_connected.return_value = False
    assert rcon.is_connected() is False, "Should be disconnected after connection loss"


def test_enter_exit(rcon: SyncRcon, connection: mock.Mock) -> None:
    assert rcon._loop is None, "Initial connection should be None"

    with rcon.connect():
        assert rcon._loop is not None, "Loop should be created"
        assert rcon._loop.is_running(), "Loop should be running"

    connection.disconnect.assert_called_once()
    assert rcon._loop is None, "Loop should be reset to None after exit"

    connection.disconnect.reset_mock()

    with contextlib.suppress(RuntimeError), rcon.connect():
        assert rcon._loop is not None, "Loop should be created again"
        assert rcon._loop.is_running(), "Loop should be running again"

        # Raise error this time
        raise RuntimeError

    connection.disconnect.assert_called_once()
    assert rcon._loop is None, "Loop should be reset to None after error"


def test_disconnect(rcon: SyncRcon) -> None:
    rcon.wait_until_connected()
    assert rcon.is_connected()

    rcon.disconnect()
    assert not rcon.is_connected()

    rcon.disconnect()
    assert not rcon.is_connected()


def test_thread_start_failure(monkeypatch: pytest.MonkeyPatch, rcon: SyncRcon) -> None:
    loop_type = type(asyncio.get_event_loop())
    monkeypatch.setattr(loop_type, "run_forever", lambda _: None)
    with pytest.raises(RuntimeError, match="Thread never signalled back"):
        rcon.wait_until_connected()
    assert rcon._loop is None, "Loop should be None after failure"

    def set_loop_none(_: None) -> None:
        rcon._loop = None

    monkeypatch.setattr(loop_type, "run_forever", set_loop_none)
    with pytest.raises(RuntimeError, match="Thread never signalled back"):
        rcon.wait_until_connected()
    assert rcon._loop is None, "Loop should be None after failure"


def test_execute_concurrently(rcon: SyncRcon, connection: mock.Mock) -> None:
    with rcon.connect():
        connection.execute.side_effect = lambda x, *_: {"ping": "pong", "foo": "bar"}[x]
        results = concurrent.futures.wait(
            (
                rcon.execute_concurrently("ping", 1),
                rcon.execute_concurrently("foo", 1),
            ),
        )
        assert {r.result() for r in results.done} == {"pong", "bar"}



import array
import asyncio
import base64
import itertools
import logging
import struct
from collections.abc import Callable
from typing import Any, Self

from typing_extensions import override

from hllrcon.exceptions import (
    HLLAuthError,
    HLLConnectionError,
    HLLConnectionLostError,
    HLLConnectionRefusedError,
    HLLMessageError,
)
from hllrcon.protocol.constants import RESPONSE_HEADER_FORMAT
from hllrcon.protocol.request import RconRequest
from hllrcon.protocol.response import RconResponse

DEFAULT_LOGGER = logging.getLogger(__name__)


class RconProtocol(asyncio.Protocol):
    """Implementation of the RCON protocol for Hell Let Loose.

    This class extends the TCP protocol to handle communication with a
    Hell Let Loose server using the RCON protocol, including sending commands
    and receiving responses.

    Example usage:
    ```python
    conn = await RconProtocol.connect(host=..., port=..., password=...)
    response = await conn.execute(
        command="KickPlayer",
        version=2,
        content_body={
            "PlayerId": "75670000000000000",
            "Reason": "Violation of rules",
        },
    )
    conn.disconnect()
    ```

    You likely do not want to use this class directly. Instead, use `RconConnection`
    which provides a higher-level interface for interacting with the game server.
    """

    def __init__(
        self,
        loop: asyncio.AbstractEventLoop,
        timeout: float | None = None,
        logger: logging.Logger | None = None,
        on_connection_lost: Callable[[Exception | None], Any] | None = None,
    ) -> None:
        """Initialize a RconProtocol instance.

        Do not initialize this class directly. Use the `connect` class method instead.

        Parameters
        ----------
        loop : asyncio.AbstractEventLoop
            The event loop to use for asynchronous operations.
        timeout : float | None, optional
            The timeout for operations, in seconds. If None, no timeout is set.
        logger : logging.Logger | None, optional
            A logger instance for logging messages. If None, a default logger is used.
        on_connection_lost : Callable[[Exception | None], Any] | None, optional
            A callback function to call when the connection is lost. It receives an
            exception if the connection was lost due to an error, or None if it was
            closed gracefully.

        """
        self._transport: asyncio.Transport | None = None
        self._buffer: bytes = b""

        self._waiters: dict[int, asyncio.Future[RconResponse]] = {}

        self.loop = loop
        self.timeout = timeout
        self.logger = logger or DEFAULT_LOGGER
        self.on_connection_lost = on_connection_lost

        self.xorkey: bytes | None = None
        self.auth_token: str | None = None

        self._counter = itertools.count(start=0)

    @classmethod
    async def connect(
        cls: type[Self],
        host: str,
        port: int,
        password: str,
        timeout: float | None = 10,
        loop: asyncio.AbstractEventLoop | None = None,
        logger: logging.Logger | None = None,
        on_connection_lost: Callable[[Exception | None], Any] | None = None,
    ) -> Self:
        """Establish a connection to the Hell Let Loose server.

        This method creates a connection to the specified server and authenticates
        using the provided password. It returns an instance of the RconProtocol.

        Parameters
        ----------
        host : str
            The hostname or IP address of the Hell Let Loose server.
        port : int
            The port number on which the RCON server is listening.
        password : str
            The RCON password for authentication.
        timeout : float | None, optional
            The timeout for the connection attempt, in seconds, by default 10.
        loop : asyncio.AbstractEventLoop | None, optional
            The event loop to use for asynchronous operations, by default None. If None,
            `asyncio.get_running_loop()` is used.
        logger : logging.Logger | None, optional
            A logger instance for logging messages, by default None. If None,
            `logging.getLogger(__name__)` is used.
        on_connection_lost : Callable[[Exception | None], Any] | None, optional
            An optional callback function to call when the connection is lost, by
            default None.

        Raises
        ------
        HLLConnectionError
            The address and port could not be resolved.
        HLLConnectionRefusedError
            The server refused the connection.
        HLLAuthError
            The provided password is incorrect.

        """
        loop = loop or asyncio.get_running_loop()

        def protocol_factory() -> Self:  # type: ignore[type-var, misc]
            return cls(  # pragma: no cover
                loop=loop,
                timeout=timeout,
                logger=logger,
                on_connection_lost=on_connection_lost,
            )

        try:
            self: Self
            _, self = await asyncio.wait_for(
                loop.create_connection(protocol_factory, host=host, port=port),
                timeout=15,
            )
        except TimeoutError:
            msg = f"Address {host} could not be resolved"
            raise HLLConnectionError(msg) from None
        except ConnectionRefusedError:
            msg = f"The server refused connection over port {port}"
            raise HLLConnectionRefusedError(msg) from None

        self.logger.info("Connected!")

        try:
            await self.authenticate(password)
        except HLLAuthError:
            self.disconnect()
            raise

        return self

    def disconnect(self) -> None:
        """Close the connection to the Hell Let Loose server.

        If the connection is already closed, this method does nothing.
        """
        if self._transport:
            self._transport.close()
        self._transport = None

    def is_connected(self) -> bool:
        """Check if the protocol is connected to the server.

        Returns
        -------
        bool
            True if the protocol is connected, False otherwise.

        """
        return self._transport is not None and not self._transport.is_closing()

    @override
    def connection_made(self, transport: asyncio.BaseTransport) -> None:
        self.logger.info("Connection made! Transport: %s", transport)
        if not isinstance(transport, asyncio.Transport):
            msg = "Transport must be an instance of asyncio.Transport"
            raise TypeError(msg)
        self._transport = transport

    @override
    def data_received(self, data: bytes) -> None:
        self.logger.debug("Incoming: (%s) %s", self._xor(data).count(b"\t"), data[:10])

        self._buffer += data
        self._read_from_buffer()

    def _read_from_buffer(self) -> None:
        pkt_id: int
        pkt_len: int

        # Read header
        header_len = struct.calcsize(RESPONSE_HEADER_FORMAT)
        if len(self._buffer) < header_len:
            self.logger.debug(
                "Buffer too small (%s < %s)",
                len(self._buffer),
                header_len,
            )
            return
        pkt_id, pkt_len = struct.unpack(
            RESPONSE_HEADER_FORMAT,
            self._buffer[:header_len],
        )
        pkt_size = header_len + pkt_len
        self.logger.debug("pkt_id = %s, pkt_len = %s", pkt_id, pkt_len)

        # Check whether whole packet is on buffer
        if len(self._buffer) >= pkt_size:
            # Read packet data from buffer
            decoded_body = self._xor(self._buffer[header_len:pkt_size])
            self.logger.debug("Unpacking: %s", decoded_body)
            pkt = RconResponse.unpack(pkt_id, decoded_body)
            self._buffer = self._buffer[pkt_size:]

            # Respond to waiter
            waiter = self._waiters.pop(pkt_id, None)
            if not waiter:
                self.logger.warning(
                    "No waiter for packet with ID %s, %s",
                    pkt_id,
                    self._waiters,
                )
            else:
                waiter.set_result(pkt)

            # Repeat if buffer is not empty; Another complete packet might be on it
            if self._buffer:
                self._read_from_buffer()

    @override
    def connection_lost(self, exc: Exception | None) -> None:
        self._transport = None

        waiters = list(self._waiters.values())
        self._waiters.clear()

        if exc:
            self.logger.warning("Connection lost: %s", exc)
            for waiter in waiters:
                if not waiter.done():
                    waiter.set_exception(HLLConnectionLostError(str(exc)))

        else:
            self.logger.info("Connection closed")
            for waiter in waiters:
                waiter.cancel()

        if self.on_connection_lost:
            try:
                self.on_connection_lost(exc)
            except Exception:
                self.logger.exception("Failed to invoke on_connection_lost hook")

    def _xor(self, message: bytes, offset: int = 0) -> bytes:
        """Encrypt or decrypt a message using the XOR key provided by the server.

        Parameters
        ----------
        message : bytes
            The message to encrypt or decrypt.
        offset : int, optional
            The offset to apply to the XOR key. Defaults to 0.

        Returns
        -------
        bytes
            The encrypted or decrypted message.

        """
        if not self.xorkey:
            return message

        n = [
            c ^ self.xorkey[(i + offset) % len(self.xorkey)]
            for i, c in enumerate(message)
        ]

        res = array.array("B", n).tobytes()
        if len(res) != len(message):
            self.logger.warning(
                "XOR operation resulted in a different length: %s != %s",
                len(res),
                len(message),
            )
            msg = "XOR operation resulted in a different length"
            raise ValueError(msg)

        return res

    async def execute(
        self,
        command: str,
        version: int,
        content_body: dict[str, Any] | str = "",
    ) -> RconResponse:
        """Execute a RCON command.

        Sends a request to the server and waits for a response.

        Parameters
        ----------
        command : str
            The command to execute on the server.
        version : int
            The version of the command.
        content_body : dict[str, Any] | str, optional
            An additional payload to send along with the command. Must be
            JSON-serializable.

        Raises
        ------
        HLLConnectionError
            The connection was closed
        HLLCommandError
            The server failed to execute the command
        HLLMessageError
            The server returned an unexpected response

        """
        if not self._transport:
            msg = "Connection is closed"
            raise HLLConnectionError(msg)

        # Create request
        request = RconRequest(
            command=command,
            version=version,
            auth_token=self.auth_token,
            content_body=content_body,
        )
        # Temporary solution to ensure each connection uses its own counter
        request.request_id = next(self._counter)

        # Send request
        header, body = request.pack()
        message = header + self._xor(body)
        self.logger.debug("Writing: %s", header + body)
        self._transport.write(message)

        try:
            # Create waiter for response
            waiter: asyncio.Future[RconResponse] = self.loop.create_future()
            self._waiters[request.request_id] = waiter

            # Wait for response
            response = await asyncio.wait_for(waiter, timeout=self.timeout)
            self.logger.debug(
                "Response: (%s) %s",
                response.name,
                response.content_body,
            )
            return response
        finally:
            # Cleanup waiter
            waiter.cancel()
            self._waiters.pop(request.request_id, None)

    async def authenticate(self, password: str) -> None:
        """Authenticate with the Hell Let Loose server.

        Parameters
        ----------
        password : str
            The RCON password to authenticate with.

        Raises
        ------
        HLLAuthError
            The provided password is incorrect.

        """
        self.logger.debug("Waiting to login...")

        xorkey_resp = await self.execute("ServerConnect", 2, "")
        xorkey_resp.raise_for_status()
        self.logger.info("Received xorkey")

        if not isinstance(xorkey_resp.content_body, str):
            msg = "ServerConnect response content_body is not a string"
            raise HLLMessageError(msg)
        self.xorkey = base64.b64decode(xorkey_resp.content_body)

        auth_token_resp = await self.execute("Login", 2, password)
        auth_token_resp.raise_for_status()
        self.logger.info("Received auth token, successfully authenticated")

        self.auth_token = auth_token_resp.content_body



import asyncio
import logging
from collections.abc import AsyncGenerator
from contextlib import asynccontextmanager
from typing import Any

from typing_extensions import override

from hllrcon.client import RconClient
from hllrcon.connection import RconConnection


class Rcon(RconClient):
    """An inferface for connecting to an RCON server.

    This class will (re)connect to the RCON server on-demand. Only when no connection is
    available at the time of executing a command will a new connection be attempted to
    be established.
    """

    def __init__(
        self,
        host: str,
        port: int,
        password: str,
        logger: logging.Logger | None = None,
        reconnect_after_failures: int = 3,
    ) -> None:
        """Initialize a new `Rcon` instance.

        Parameters
        ----------
        host : str
            The hostname or IP address of the RCON server.
        port : int
            The port of the RCON server.
        password : str
            The password for the RCON server.
        logger : logging.Logger | None, optional
            A logger instance for logging messages, by default None. If None,
            `logging.getLogger(__name__)` is used.
        reconnect_after_failures : int, optional
            After how many failed attempts to execute a command the active connection is
            disposed and a new connection is established on the next command execution.
            If the server responds to a request, the failure count is reset, even if the
            server returned an error. Set to 0 to disable, by default 3.

        """
        super().__init__()
        self.host = host
        self.port = port
        self.password = password
        self.reconnect_after_failures = max(0, reconnect_after_failures)

        self._logger = logger
        self._connection: asyncio.Future[RconConnection] | None = None
        self._failure_count = 0

    @property
    def logger(self) -> logging.Logger:
        return self._logger or logging.getLogger(__name__)

    @logger.setter
    def logger(self, value: logging.Logger | None) -> None:
        self._logger = value

    async def _get_connection(self) -> RconConnection:
        if (
            self._connection
            and self._connection.done()
            and (
                self._connection.exception()
                or not self._connection.result().is_connected()
            )
        ):
            self._connection = None

        if self._connection is None:
            self._connection = asyncio.Future()
            try:
                connection = await RconConnection.connect(
                    host=self.host,
                    port=self.port,
                    password=self.password,
                    logger=self._logger,
                )
                self._connection.set_result(connection)
            except Exception as e:
                self._connection.set_exception(e)
                self._connection = None
                raise
            else:
                return connection

        elif not self._connection.done():
            return await asyncio.shield(self._connection)

        else:
            return self._connection.result()

    @override
    def is_connected(self) -> bool:
        return (
            self._connection is not None
            and not self._connection.cancelled()
            and self._connection.done()
            and not self._connection.exception()
            and self._connection.result().is_connected()
        )

    @override
    @asynccontextmanager
    async def connect(self) -> AsyncGenerator[None]:
        await self._get_connection()
        try:
            yield
        finally:
            self.disconnect()

    @override
    async def wait_until_connected(self) -> None:
        await self._get_connection()

    @override
    def disconnect(self) -> None:
        if self._connection:
            if self._connection.done():
                if not self._connection.exception():
                    self._connection.result().disconnect()
            else:
                self._connection.cancel()

        self._connection = None
        self._failure_count = 0

    @override
    async def execute(
        self,
        command: str,
        version: int,
        body: str | dict[str, Any] = "",
    ) -> str:
        connection = await self._get_connection()

        try:
            return await connection.execute(command, version, body)
        except (TimeoutError, OSError):
            self._failure_count += 1
            if (
                self.reconnect_after_failures > 0
                and self._failure_count >= self.reconnect_after_failures
            ):
                self.disconnect()
            raise



from hllrcon.protocol.request import RconRequest


def test_rconrequest_init_sets_fields() -> None:
    req = RconRequest("mycmd", 2, "token123", {"foo": "bar"})
    assert req.name == "mycmd"
    assert req.version == 2
    assert req.auth_token == "token123"
    assert req.content_body == {"foo": "bar"}
    assert isinstance(req.request_id, int)


def test_rconrequest_id_is_unique() -> None:
    req1 = RconRequest("cmd1", 1, None)
    req2 = RconRequest("cmd2", 1, None)
    assert req1.request_id != req2.request_id


def test_pack_with_header() -> None:
    req = RconRequest("cmd", 1, "tok", {"z": 9})
    packed = req.pack()

    expected_body = (
        b'{"authToken":"tok","version":1,"name":"cmd","contentBody":"{\\"z\\":9}"}'
    )
    expected_header = req.request_id.to_bytes(4, byteorder="little") + len(
        expected_body,
    ).to_bytes(4, byteorder="little")
    assert packed[0] == expected_header
    assert packed[1] == expected_body



"""Script to generate a JSON schema of all available RCON commands for the HLL server.

To run this script, ensure the `HLL_HOST`, `HLL_PORT`, and `HLL_PASSWORD` environment
variables are set.
Then, run `uv run scripts/generate_command_schema.py` in your terminal. The output will
be saved to `commands_schema.json`.
"""

import asyncio
import logging

import aiofiles
from pydantic import RootModel

from hllrcon.connection import RconConnection
from hllrcon.responses import GetCommandDetailsResponse
from scripts import HLL_SERVER_CREDENTIALS

logging.basicConfig(level=logging.INFO)


AllCommandDetails = RootModel[list[GetCommandDetailsResponse]]


async def main() -> None:
    if not HLL_SERVER_CREDENTIALS:
        msg = "Server credentials must be set in the environment variables."
        raise ValueError(msg)

    conn = await RconConnection.connect(**HLL_SERVER_CREDENTIALS)

    all_commands = await conn.get_commands()
    all_command_details = AllCommandDetails(
        await asyncio.gather(
            *(conn.get_command_details(entry.id) for entry in all_commands.entries),
        ),
    )

    async with aiofiles.open("commands_schema.json", "w", encoding="utf-8") as f:
        content = all_command_details.model_dump_json(indent=2)
        await f.write(content)


if __name__ == "__main__":
    asyncio.run(main())



# ruff: noqa: N802, D400, D415

from enum import StrEnum
from functools import cached_property
from typing import TYPE_CHECKING

from pydantic import Field

from hllrcon.data._utils import IndexedBaseModel, class_cached_property
from hllrcon.data.factions import Faction

if TYPE_CHECKING:
    from hllrcon.data.vehicles import Vehicle


class WeaponType(StrEnum):
    BOLT_ACTION_RIFLE = "Bolt Action Rifle"
    SEMI_AUTO_RIFLE = "Semi-Auto Rifle"
    ASSAULT_RIFLE = "Assault Rifle"
    SUBMACHINE_GUN = "Submachine Gun"
    MACHINE_GUN = "Machine Gun"
    SHOTGUN = "Shotgun"
    PISTOL = "Pistol"
    REVOLVER = "Revolver"
    GRENADE = "Grenade"
    AP_MINE = "Anti-Personnel Mine"
    AT_MINE = "Anti-Tank Mine"
    FLAMETHROWER = "Flamethrower"
    MELEE = "Melee"
    FLARE_GUN = "Flare Gun"
    ROCKET_LAUNCHER = "Rocket Launcher"
    ANTI_MATERIEL_RIFLE = "Anti-Materiel Rifle"
    AT_GUN = "Anti-Tank Gun"
    TANK_CANNON = "Tank Cannon"
    TANK_COAXIAL_MG = "Tank Coaxial MG"
    TANK_HULL_MG = "Tank Hull MG"
    MOUNTED_MG = "Mounted MG"
    ROADKILL = "Roadkill"
    ARTILLERY = "Artillery"
    COMMANDER_ABILITY = "Commander Ability"
    SATCHEL = "Satchel"
    UNKNOWN = "Unknown"


class Weapon(IndexedBaseModel[str]):
    id: str
    name: str
    type: WeaponType
    vehicle_id: str | None = None
    factions: set[Faction] = Field(min_length=1)
    magnification: int | None = None

    # @computed_field(repr=False)  # type: ignore[prop-decorator]
    @cached_property
    def vehicle(self) -> "Vehicle | None":
        from hllrcon.data.vehicles import Vehicle  # noqa: PLC0415

        if self.vehicle_id:
            return Vehicle.by_id(self.vehicle_id)
        return None

    # --- American weapons ---

    @class_cached_property
    @classmethod
    def M1_GARAND(cls) -> "Weapon":
        """*M1 GARAND*"""
        return cls(
            id="M1 GARAND",
            name="M1 Garand",
            factions={Faction.US},
            type=WeaponType.SEMI_AUTO_RIFLE,
        )

    @class_cached_property
    @classmethod
    def M1_CARBINE(cls) -> "Weapon":
        """*M1 CARBINE*"""
        return cls(
            id="M1 CARBINE",
            name="M1 Carbine",
            factions={Faction.US},
            type=WeaponType.SEMI_AUTO_RIFLE,
        )

    @class_cached_property
    @classmethod
    def M1A1_THOMPSON(cls) -> "Weapon":
        """*M1A1 THOMPSON*"""
        return cls(
            id="M1A1 THOMPSON",
            name="M1A1 Thompson",
            factions={Faction.US},
            type=WeaponType.SUBMACHINE_GUN,
        )

    @class_cached_property
    @classmethod
    def M3_GREASE_GUN(cls) -> "Weapon":
        """*M3 GREASE GUN*"""
        return cls(
            id="M3 GREASE GUN",
            name="M3 Grease Gun",
            factions={Faction.US},
            type=WeaponType.SUBMACHINE_GUN,
        )

    @class_cached_property
    @classmethod
    def M1918A2_BAR(cls) -> "Weapon":
        """*M1918A2 BAR*"""
        return cls(
            id="M1918A2 BAR",
            name="M1918A2 BAR",
            factions={Faction.US},
            type=WeaponType.ASSAULT_RIFLE,
        )

    @class_cached_property
    @classmethod
    def BROWNING_M1919(cls) -> "Weapon":
        """*BROWNING M1919*"""
        return cls(
            id="BROWNING M1919",
            name="M1919 Browning",
            factions={Faction.US},
            type=WeaponType.MACHINE_GUN,
        )

    @class_cached_property
    @classmethod
    def M1903_SPRINGFIELD_SCOPED_4X(cls) -> "Weapon":
        """*M1903 SPRINGFIELD*"""
        return cls(
            id="M1903 SPRINGFIELD",
            name="M1903 Springfield",
            factions={Faction.US},
            type=WeaponType.BOLT_ACTION_RIFLE,
            magnification=4,
        )

    @class_cached_property
    @classmethod
    def M97_TRENCH_GUN(cls) -> "Weapon":
        """*M97 TRENCH GUN*"""
        return cls(
            id="M97 TRENCH GUN",
            name="M97 Trench Gun",
            factions={Faction.US},
            type=WeaponType.SHOTGUN,
        )

    @class_cached_property
    @classmethod
    def COLT_M1911(cls) -> "Weapon":
        """*COLT M1911*"""
        return cls(
            id="COLT M1911",
            name="Colt M1911",
            factions={Faction.US},
            type=WeaponType.PISTOL,
        )

    @class_cached_property
    @classmethod
    def M3_KNIFE(cls) -> "Weapon":
        """*M3 KNIFE*"""
        return cls(
            id="M3 KNIFE",
            name="M3 Knife",
            factions={Faction.US},
            type=WeaponType.MELEE,
        )

    @class_cached_property
    @classmethod
    def SATCHEL_CHARGE(cls) -> "Weapon":
        """*SATCHEL*"""
        return cls(
            id="SATCHEL",
            name="Satchel Charge",
            factions={Faction.US, Faction.GER, Faction.DAK},
            type=WeaponType.SATCHEL,
        )

    @class_cached_property
    @classmethod
    def MK2_GRENADE(cls) -> "Weapon":
        """*MK2 GRENADE*"""
        return cls(
            id="MK2 GRENADE",
            name="Mk 2 Grenade",
            factions={Faction.US},
            type=WeaponType.GRENADE,
        )

    @class_cached_property
    @classmethod
    def M2_FLAMETHROWER(cls) -> "Weapon":
        """*M2 FLAMETHROWER*"""
        return cls(
            id="M2 FLAMETHROWER",
            name="M2 Flamethrower",
            factions={Faction.US},
            type=WeaponType.FLAMETHROWER,
        )

    @class_cached_property
    @classmethod
    def BAZOOKA(cls) -> "Weapon":
        """*BAZOOKA*"""
        return cls(
            id="BAZOOKA",
            name="Bazooka",
            factions={Faction.US},
            type=WeaponType.ROCKET_LAUNCHER,
        )

    @class_cached_property
    @classmethod
    def M2_AP_MINE(cls) -> "Weapon":
        """*M2 AP MINE*"""
        return cls(
            id="M2 AP MINE",
            name="M2 AP Mine",
            factions={Faction.US},
            type=WeaponType.AP_MINE,
        )

    @class_cached_property
    @classmethod
    def M1A1_AT_MINE(cls) -> "Weapon":
        """*M1A1 AT MINE*"""
        return cls(
            id="M1A1 AT MINE",
            name="M1A1 AT Mine",
            factions={Faction.US},
            type=WeaponType.AT_MINE,
        )

    @class_cached_property
    @classmethod
    def FLARE_GUN(cls) -> "Weapon":
        """*FLARE GUN*"""
        return cls(
            id="FLARE GUN",
            name="Flare Gun",
            factions={Faction.US, Faction.GER, Faction.DAK, Faction.SOV},
            type=WeaponType.FLARE_GUN,
        )

    @class_cached_property
    @classmethod
    def V_57MM_CANNON__M1_57(cls) -> "Weapon":
        """*57MM CANNON [M1 57mm]*"""
        return cls(
            id="57MM CANNON [M1 57mm]",
            name="57mm Cannon",
            vehicle_id="M1 57mm",
            factions={Faction.US},
            type=WeaponType.AT_GUN,
        )

    @class_cached_property
    @classmethod
    def V_155MM_HOWITZER__M114(cls) -> "Weapon":
        """*155MM HOWITZER [M114]*"""
        return cls(
            id="155MM HOWITZER [M114]",
            name="155mm Howitzer",
            vehicle_id="M114",
            factions={Faction.US},
            type=WeaponType.ARTILLERY,
        )

    @class_cached_property
    @classmethod
    def V_ROADKILL__M8_GREYHOUND(cls) -> "Weapon":
        """*M8 Greyhound*"""
        return cls(
            id="M8 Greyhound",
            name="Roadkill",
            vehicle_id="M8 Greyhound",
            factions={Faction.US},
            type=WeaponType.ROADKILL,
        )

    @class_cached_property
    @classmethod
    def V_ROADKILL__STUART_M5A1(cls) -> "Weapon":
        """*Stuart M5A1*"""
        return cls(
            id="Stuart M5A1",
            name="Roadkill",
            vehicle_id="Stuart M5A1",
            factions={Faction.US},
            type=WeaponType.ROADKILL,
        )

    @class_cached_property
    @classmethod
    def V_ROADKILL__SHERMAN_M4A3_75W(cls) -> "Weapon":
        """*Sherman M4A3(75)W*"""
        return cls(
            id="Sherman M4A3(75)W",
            name="Roadkill",
            vehicle_id="Sherman M4A3(75)W",
            factions={Faction.US},
            type=WeaponType.ROADKILL,
        )

    @class_cached_property
    @classmethod
    def V_ROADKILL__SHERMAN_M4A3E2(cls) -> "Weapon":
        """*Sherman M4A3E2*"""
        return cls(
            id="Sherman M4A3E2",
            name="Roadkill",
            vehicle_id="Sherman M4A3E2",
            factions={Faction.US},
            type=WeaponType.ROADKILL,
        )

    @class_cached_property
    @classmethod
    def V_ROADKILL__SHERMAN_M4A3E2_76(cls) -> "Weapon":
        """*Sherman M4A3E2(76)*"""
        return cls(
            id="Sherman M4A3E2(76)",
            name="Roadkill",
            vehicle_id="Sherman M4A3E2(76)",
            factions={Faction.US},
            type=WeaponType.ROADKILL,
        )

    @class_cached_property
    @classmethod
    def V_ROADKILL__GMC_CCKW_353_SUPPLY(cls) -> "Weapon":
        """*GMC CCKW 353 (Supply)*"""
        return cls(
            id="GMC CCKW 353 (Supply)",
            name="Roadkill",
            vehicle_id="GMC CCKW 353 (Supply)",
            factions={Faction.US},
            type=WeaponType.ROADKILL,
        )

    @class_cached_property
    @classmethod
    def V_ROADKILL__GMC_CCKW_353_TRANSPORT(cls) -> "Weapon":
        """*GMC CCKW 353 (Transport)*"""
        return cls(
            id="GMC CCKW 353 (Transport)",
            name="Roadkill",
            vehicle_id="GMC CCKW 353 (Transport)",
            factions={Faction.US},
            type=WeaponType.ROADKILL,
        )

    @class_cached_property
    @classmethod
    def V_ROADKILL__M3_HALF_TRACK(cls) -> "Weapon":
        """*M3 Half-track*"""
        return cls(
            id="M3 Half-track",
            name="Roadkill",
            vehicle_id="M3 Half-track",
            factions={Faction.US, Faction.SOV, Faction.CW, Faction.B8A},
            type=WeaponType.ROADKILL,
        )

    @class_cached_property
    @classmethod
    def V_ROADKILL__JEEP_WILLYS(cls) -> "Weapon":
        """*Jeep Willys*"""
        return cls(
            id="Jeep Willys",
            name="Roadkill",
            vehicle_id="Jeep Willys",
            factions={Faction.US},
            type=WeaponType.ROADKILL,
        )

    @class_cached_property
    @classmethod
    def V_M6_37MM__M8_GREYHOUND(cls) -> "Weapon":
        """*M6 37mm [M8 Greyhound]*"""
        return cls(
            id="M6 37mm [M8 Greyhound]",
            name="37mm Cannon",
            vehicle_id="M8 Greyhound",
            factions={Faction.US},
            type=WeaponType.TANK_CANNON,
        )

    @class_cached_property
    @classmethod
    def V_COAXIAL_M1919__M8_GREYHOUND(cls) -> "Weapon":
        """*COAXIAL M1919 [M8 Greyhound]*"""
        return cls(
            id="COAXIAL M1919 [M8 Greyhound]",
            name="M1919 Browning",
            vehicle_id="M8 Greyhound",
            factions={Faction.US},
            type=WeaponType.TANK_COAXIAL_MG,
        )

    @class_cached_property
    @classmethod
    def V_37MM_CANNON__STUART_M5A1(cls) -> "Weapon":
        """*37MM CANNON [Stuart M5A1]*"""
        return cls(
            id="37MM CANNON [Stuart M5A1]",
            name="37mm Cannon",
            vehicle_id="Stuart M5A1",
            factions={Faction.US},
            type=WeaponType.TANK_CANNON,
        )

    @class_cached_property
    @classmethod
    def V_COAXIAL_M1919__STUART_M5A1(cls) -> "Weapon":
        """*COAXIAL M1919 [Stuart M5A1]*"""
        return cls(
            id="COAXIAL M1919 [Stuart M5A1]",
            name="M1919 Browning",
            vehicle_id="Stuart M5A1",
            factions={Faction.US},
            type=WeaponType.TANK_COAXIAL_MG,
        )

    @class_cached_property
    @classmethod
    def V_HULL_M1919__STUART_M5A1(cls) -> "Weapon":
        """*HULL M1919 [Stuart M5A1]*"""
        return cls(
            id="HULL M1919 [Stuart M5A1]",
            name="M1919 Browning",
            vehicle_id="Stuart M5A1",
            factions={Faction.US},
            type=WeaponType.TANK_HULL_MG,
        )

    @class_cached_property
    @classmethod
    def V_75MM_CANNON__SHERMAN_M4A3_75W(cls) -> "Weapon":
        """*75MM CANNON [Sherman M4A3(75)W]*"""
        return cls(
            id="75MM CANNON [Sherman M4A3(75)W]",
            name="75mm Cannon",
            vehicle_id="Sherman M4A3(75)W",
            factions={Faction.US},
            type=WeaponType.TANK_CANNON,
        )

    @class_cached_property
    @classmethod
    def V_COAXIAL_M1919__SHERMAN_M4A3_75W(cls) -> "Weapon":
        """*COAXIAL M1919 [Sherman M4A3(75)W]*"""
        return cls(
            id="COAXIAL M1919 [Sherman M4A3(75)W]",
            name="M1919 Browning",
            vehicle_id="Sherman M4A3(75)W",
            factions={Faction.US},
            type=WeaponType.TANK_COAXIAL_MG,
        )

    @class_cached_property
    @classmethod
    def V_HULL_M1919__SHERMAN_M4A3_75W(cls) -> "Weapon":
        """`HULL M1919 [Sherman M4A3(75)W]`"""
        return cls(
            id="HULL M1919 [Sherman M4A3(75)W]",
            name="M1919 Browning",
            vehicle_id="Sherman M4A3(75)W",
            factions={Faction.US},
            type=WeaponType.TANK_HULL_MG,
        )

    @class_cached_property
    @classmethod
    def V_75MM_M3_GUN__SHERMAN_M4A3E2(cls) -> "Weapon":
        """*75MM M3 GUN [Sherman M4A3E2]*"""
        return cls(
            id="75MM M3 GUN [Sherman M4A3E2]",
            name="75mm Cannon",
            vehicle_id="Sherman M4A3E2",
            factions={Faction.US},
            type=WeaponType.TANK_CANNON,
        )

    @class_cached_property
    @classmethod
    def V_COAXIAL_M1919__SHERMAN_M4A3E2(cls) -> "Weapon":
        """*COAXIAL M1919 [Sherman M4A3E2]*"""
        return cls(
            id="COAXIAL M1919 [Sherman M4A3E2]",
            name="M1919 Browning",
            vehicle_id="Sherman M4A3E2",
            factions={Faction.US},
            type=WeaponType.TANK_COAXIAL_MG,
        )

    @class_cached_property
    @classmethod
    def V_HULL_M1919__SHERMAN_M4A3E2(cls) -> "Weapon":
        """*HULL M1919 [Sherman M4A3E2]*"""
        return cls(
            id="HULL M1919 [Sherman M4A3E2]",
            name="M1919 Browning",
            vehicle_id="Sherman M4A3E2",
            factions={Faction.US},
            type=WeaponType.TANK_HULL_MG,
        )

    @class_cached_property
    @classmethod
    def V_76MM_M1_GUN__SHERMAN_M4A3E2_76(cls) -> "Weapon":
        """*76MM M1 GUN [Sherman M4A3E2(76)]*"""
        return cls(
            id="76MM M1 GUN [Sherman M4A3E2(76)]",
            name="76mm Cannon",
            vehicle_id="Sherman M4A3E2(76)",
            factions={Faction.US},
            type=WeaponType.TANK_CANNON,
        )

    @class_cached_property
    @classmethod
    def V_COAXIAL_M1919__SHERMAN_M4A3E2_76(cls) -> "Weapon":
        """*COAXIAL M1919 [Sherman M4A3E2(76)]*"""
        return cls(
            id="COAXIAL M1919 [Sherman M4A3E2(76)]",
            name="M1919 Browning",
            vehicle_id="Sherman M4A3E2(76)",
            factions={Faction.US},
            type=WeaponType.TANK_COAXIAL_MG,
        )

    @class_cached_property
    @classmethod
    def V_HULL_M1919__SHERMAN_M4A3E2_76(cls) -> "Weapon":
        """*HULL M1919 [Sherman M4A3E2(76)]*"""
        return cls(
            id="HULL M1919 [Sherman M4A3E2(76)]",
            name="M1919 Browning",
            vehicle_id="Sherman M4A3E2(76)",
            factions={Faction.US},
            type=WeaponType.TANK_HULL_MG,
        )

    @class_cached_property
    @classmethod
    def V_M2_BROWNING__M3_HALF_TRACK(cls) -> "Weapon":
        """*M2 Browning [M3 Half-track]*"""
        return cls(
            id="M2 Browning [M3 Half-track]",
            name="M2 Browning",
            vehicle_id="M3 Half-track",
            factions={Faction.US, Faction.SOV, Faction.CW, Faction.B8A},
            type=WeaponType.MOUNTED_MG,
        )

    @class_cached_property
    @classmethod
    def V_57MM_CANNON__UNKNOWN(cls) -> "Weapon":
        """*57MM CANNON*"""
        return cls(
            id="57MM CANNON",
            name="57mm Cannon",
            vehicle_id=None,
            factions={Faction.US, Faction.SOV},
            type=WeaponType.AT_GUN,
        )

    @class_cached_property
    @classmethod
    def V_76MM_M1_GUN__UNKNOWN(cls) -> "Weapon":
        """*76MM M1 GUN*"""
        return cls(
            id="76MM M1 GUN",
            name="76mm Cannon",
            vehicle_id=None,
            factions={Faction.US},
            type=WeaponType.TANK_CANNON,
        )

    @class_cached_property
    @classmethod
    def V_75MM_M3_GUN__UNKNOWN(cls) -> "Weapon":
        """*75MM M3 GUN*"""
        return cls(
            id="75MM M3 GUN",
            name="75mm Cannon",
            vehicle_id=None,
            factions={Faction.US},
            type=WeaponType.TANK_CANNON,
        )

    @class_cached_property
    @classmethod
    def V_75MM_CANNON__UNKNOWN(cls) -> "Weapon":
        """*75MM CANNON*"""
        return cls(
            id="75MM CANNON",
            name="75mm Cannon",
            vehicle_id=None,
            factions={Faction.US, Faction.GER, Faction.DAK},
            type=WeaponType.UNKNOWN,
        )

    @class_cached_property
    @classmethod
    def V_37MM_CANNON__UNKNOWN(cls) -> "Weapon":
        """*37MM CANNON*"""
        return cls(
            id="37MM CANNON",
            name="37mm Cannon",
            vehicle_id=None,
            factions={Faction.US, Faction.GB, Faction.CW},
            type=WeaponType.TANK_CANNON,
        )

    @class_cached_property
    @classmethod
    def V_M6_37MM__UNKNOWN(cls) -> "Weapon":
        """*M6 37MM*"""
        return cls(
            id="M6 37MM",
            name="M6 37mm",
            vehicle_id=None,
            factions={Faction.US},
            type=WeaponType.TANK_CANNON,
        )

    @class_cached_property
    @classmethod
    def V_COAXIAL_M1919__UNKNOWN(cls) -> "Weapon":
        """*COAXIAL M1919*"""
        return cls(
            id="COAXIAL M1919",
            name="M1919 Browning",
            vehicle_id=None,
            factions={Faction.US, Faction.GB, Faction.CW},
            type=WeaponType.TANK_COAXIAL_MG,
        )

    @class_cached_property
    @classmethod
    def V_HULL_M1919__UNKNOWN(cls) -> "Weapon":
        """*HULL M1919*"""
        return cls(
            id="HULL M1919",
            name="M1919 Browning",
            vehicle_id=None,
            factions={Faction.US, Faction.GB, Faction.CW},
            type=WeaponType.TANK_HULL_MG,
        )

    @class_cached_property
    @classmethod
    def V_M2_BROWNING__UNKNOWN(cls) -> "Weapon":
        """*M2 Browning*"""
        return cls(
            id="M2 Browning",
            name="M2 Browning",
            vehicle_id=None,
            factions={Faction.US, Faction.SOV, Faction.CW, Faction.B8A},
            type=WeaponType.MOUNTED_MG,
        )

    # --- German weapons ---

    @class_cached_property
    @classmethod
    def KARABINER_98K(cls) -> "Weapon":
        """*KARABINER 98K*"""
        return cls(
            id="KARABINER 98K",
            name="Karabiner 98k",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.BOLT_ACTION_RIFLE,
        )

    @class_cached_property
    @classmethod
    def GEWEHR_43(cls) -> "Weapon":
        """*GEWEHR 43*"""
        return cls(
            id="GEWEHR 43",
            name="G43",
            factions={Faction.GER},
            type=WeaponType.SEMI_AUTO_RIFLE,
        )

    @class_cached_property
    @classmethod
    def STG44(cls) -> "Weapon":
        """*STG44*"""
        return cls(
            id="STG44",
            name="STG44",
            factions={Faction.GER},
            type=WeaponType.ASSAULT_RIFLE,
        )

    @class_cached_property
    @classmethod
    def FG42(cls) -> "Weapon":
        """*FG42*"""
        return cls(
            id="FG42",
            name="FG42",
            factions={Faction.GER},
            type=WeaponType.ASSAULT_RIFLE,
        )

    @class_cached_property
    @classmethod
    def MP40(cls) -> "Weapon":
        """*MP40*"""
        return cls(
            id="MP40",
            name="MP40",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.SUBMACHINE_GUN,
        )

    @class_cached_property
    @classmethod
    def MG34(cls) -> "Weapon":
        """*MG34*"""
        return cls(
            id="MG34",
            name="MG34",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.MACHINE_GUN,
        )

    @class_cached_property
    @classmethod
    def MG42(cls) -> "Weapon":
        """*MG42*"""
        return cls(
            id="MG42",
            name="MG42",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.MACHINE_GUN,
        )

    @class_cached_property
    @classmethod
    def FLAMMENWERFER_41(cls) -> "Weapon":
        """*FLAMMENWERFER 41*"""
        return cls(
            id="FLAMMENWERFER 41",
            name="Flammenwerfer 41",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.FLAMETHROWER,
        )

    @class_cached_property
    @classmethod
    def KARABINER_98K_SCOPED_8X(cls) -> "Weapon":
        """*KARABINER 98K x8*"""
        return cls(
            id="KARABINER 98K x8",
            name="Karabiner 98k",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.BOLT_ACTION_RIFLE,
            magnification=8,
        )

    @class_cached_property
    @classmethod
    def FG42_SCOPED_4X(cls) -> "Weapon":
        """*FG42 x4*"""
        return cls(
            id="FG42 x4",
            name="FG42",
            factions={Faction.GER},
            type=WeaponType.SEMI_AUTO_RIFLE,
            magnification=4,
        )

    @class_cached_property
    @classmethod
    def LUGER_P08(cls) -> "Weapon":
        """*LUGER P08*"""
        return cls(
            id="LUGER P08",
            name="Luger P08",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.PISTOL,
        )

    @class_cached_property
    @classmethod
    def WALTHER_P38(cls) -> "Weapon":
        """*WALTHER P38*"""
        return cls(
            id="WALTHER P38",
            name="Walther P38",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.PISTOL,
        )

    @class_cached_property
    @classmethod
    def FELDSPATEN(cls) -> "Weapon":
        """*FELDSPATEN*"""
        return cls(
            id="FELDSPATEN",
            name="Feldspaten",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.MELEE,
        )

    # SATCHEL

    @class_cached_property
    @classmethod
    def M24_STIELHANDGRANATE(cls) -> "Weapon":
        """*M24 STIELHANDGRANATE*"""
        return cls(
            id="M24 STIELHANDGRANATE",
            name="M24 Stielhandgranate",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.GRENADE,
        )

    @class_cached_property
    @classmethod
    def M43_STIELHANDGRANATE(cls) -> "Weapon":
        """*M43 STIELHANDGRANATE*"""
        return cls(
            id="M43 STIELHANDGRANATE",
            name="M43 Stielhandgranate",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.GRENADE,
        )

    @class_cached_property
    @classmethod
    def PANZERSCHRECK(cls) -> "Weapon":
        """*PANZERSCHRECK*"""
        return cls(
            id="PANZERSCHRECK",
            name="Panzerschreck",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.ROCKET_LAUNCHER,
        )

    @class_cached_property
    @classmethod
    def S_MINE(cls) -> "Weapon":
        """*S-MINE*"""
        return cls(
            id="S-MINE",
            name="S-Mine",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.AP_MINE,
        )

    @class_cached_property
    @classmethod
    def TELLERMINE_43(cls) -> "Weapon":
        """*TELLERMINE 43*"""
        return cls(
            id="TELLERMINE 43",
            name="Tellermine 43",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.AT_MINE,
        )

    # FLARE GUN

    @class_cached_property
    @classmethod
    def V_75MM_CANNON__PAK_40(cls) -> "Weapon":
        """*75MM CANNON [PAK 40]*"""
        return cls(
            id="75MM CANNON [PAK 40]",
            name="75mm Cannon",
            vehicle_id="PAK 40",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.AT_GUN,
        )

    @class_cached_property
    @classmethod
    def V_150MM_HOWITZER__SFH_18(cls) -> "Weapon":
        """*150MM HOWITZER [sFH 18]*"""
        return cls(
            id="150MM HOWITZER [sFH 18]",
            name="150mm Howitzer",
            vehicle_id="sFH 18",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.ARTILLERY,
        )

    @class_cached_property
    @classmethod
    def V_ROADKILL__SD_KFZ_234_PUMA(cls) -> "Weapon":
        """*Sd.Kfz.234 Puma*"""
        return cls(
            id="Sd.Kfz.234 Puma",
            name="Roadkill",
            vehicle_id="Sd.Kfz.234 Puma",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.ROADKILL,
        )

    @class_cached_property
    @classmethod
    def V_ROADKILL__SD_KFZ_121_LUCHS(cls) -> "Weapon":
        """*Sd.Kfz.121 Luchs*"""
        return cls(
            id="Sd.Kfz.121 Luchs",
            name="Roadkill",
            vehicle_id="Sd.Kfz.121 Luchs",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.ROADKILL,
        )

    @class_cached_property
    @classmethod
    def V_ROADKILL__SD_KFZ_161_PANZER_IV(cls) -> "Weapon":
        """*Sd.Kfz.161 Panzer IV*"""
        return cls(
            id="Sd.Kfz.161 Panzer IV",
            name="Roadkill",
            vehicle_id="Sd.Kfz.161 Panzer IV",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.ROADKILL,
        )

    @class_cached_property
    @classmethod
    def V_ROADKILL__SD_KFZ_171_PANTHER(cls) -> "Weapon":
        """*Sd.Kfz.171 Panther*"""
        return cls(
            id="Sd.Kfz.171 Panther",
            name="Roadkill",
            vehicle_id="Sd.Kfz.171 Panther",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.ROADKILL,
        )

    @class_cached_property
    @classmethod
    def V_ROADKILL__SD_KFZ_181_TIGER_1(cls) -> "Weapon":
        """*Sd.Kfz.181 Tiger 1*"""
        return cls(
            id="Sd.Kfz.181 Tiger 1",
            name="Roadkill",
            vehicle_id="Sd.Kfz.181 Tiger 1",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.ROADKILL,
        )

    @class_cached_property
    @classmethod
    def V_ROADKILL__OPEL_BLITZ_SUPPLY(cls) -> "Weapon":
        """*Opel Blitz (Supply)*"""
        return cls(
            id="Opel Blitz (Supply)",
            name="Roadkill",
            vehicle_id="Opel Blitz (Supply)",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.ROADKILL,
        )

    @class_cached_property
    @classmethod
    def V_ROADKILL__OPEL_BLITZ_TRANSPORT(cls) -> "Weapon":
        """*Opel Blitz (Transport)*"""
        return cls(
            id="Opel Blitz (Transport)",
            name="Roadkill",
            vehicle_id="Opel Blitz (Transport)",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.ROADKILL,
        )

    @class_cached_property
    @classmethod
    def V_ROADKILL__SD_KFZ_251_HALF_TRACK(cls) -> "Weapon":
        """*Sd.Kfz 251 Half-track*"""
        return cls(
            id="Sd.Kfz 251 Half-track",
            name="Roadkill",
            vehicle_id="Sd.Kfz 251 Half-track",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.ROADKILL,
        )

    @class_cached_property
    @classmethod
    def V_ROADKILL__KUBELWAGEN(cls) -> "Weapon":
        """*Kubelwagen*"""
        return cls(
            id="Kubelwagen",
            name="Roadkill",
            vehicle_id="Kubelwagen",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.ROADKILL,
        )

    @class_cached_property
    @classmethod
    def V_50MM_KWK_91_1__SD_KFZ_234_PUMA(cls) -> "Weapon":
        """*50mm KwK 39/1 [Sd.Kfz.234 Puma]*"""
        return cls(
            id="50mm KwK 39/1 [Sd.Kfz.234 Puma]",
            name="50mm KwK 39/1",
            vehicle_id="Sd.Kfz.234 Puma",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.TANK_CANNON,
        )

    @class_cached_property
    @classmethod
    def V_COAXIAL_MG34__SD_KFZ_234_PUMA(cls) -> "Weapon":
        """*COAXIAL MG34 [Sd.Kfz.234 Puma]*"""
        return cls(
            id="COAXIAL MG34 [Sd.Kfz.234 Puma]",
            name="MG34",
            vehicle_id="Sd.Kfz.234 Puma",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.TANK_COAXIAL_MG,
        )

    @class_cached_property
    @classmethod
    def V_20MM_KWK_30__SD_KFZ_121_LUCHS(cls) -> "Weapon":
        """*20MM KWK 30 [Sd.Kfz.121 Luchs]*"""
        return cls(
            id="20MM KWK 30 [Sd.Kfz.121 Luchs]",
            name="20mm KwK 30",
            vehicle_id="Sd.Kfz.121 Luchs",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.TANK_CANNON,
        )

    @class_cached_property
    @classmethod
    def V_COAXIAL_MG34__SD_KFZ_121_LUCHS(cls) -> "Weapon":
        """*COAXIAL MG34 [Sd.Kfz.121 Luchs]*"""
        return cls(
            id="COAXIAL MG34 [Sd.Kfz.121 Luchs]",
            name="MG34",
            vehicle_id="Sd.Kfz.121 Luchs",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.TANK_COAXIAL_MG,
        )

    @class_cached_property
    @classmethod
    def V_75MM_CANNON__SD_KFZ_161_PANZER_IV(cls) -> "Weapon":
        """*75MM CANNON [Sd.Kfz.161 Panzer IV]*"""
        return cls(
            id="75MM CANNON [Sd.Kfz.161 Panzer IV]",
            name="75mm Cannon",
            vehicle_id="Sd.Kfz.161 Panzer IV",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.TANK_CANNON,
        )

    @class_cached_property
    @classmethod
    def V_COAXIAL_MG34__SD_KFZ_161_PANZER_IV(cls) -> "Weapon":
        """*COAXIAL MG34 [Sd.Kfz.161 Panzer IV]*"""
        return cls(
            id="COAXIAL MG34 [Sd.Kfz.161 Panzer IV]",
            name="MG34",
            vehicle_id="Sd.Kfz.161 Panzer IV",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.TANK_COAXIAL_MG,
        )

    @class_cached_property
    @classmethod
    def V_HULL_MG34__SD_KFZ_161_PANZER_IV(cls) -> "Weapon":
        """*HULL MG34 [Sd.Kfz.161 Panzer IV]*"""
        return cls(
            id="HULL MG34 [Sd.Kfz.161 Panzer IV]",
            name="MG34",
            vehicle_id="Sd.Kfz.161 Panzer IV",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.TANK_HULL_MG,
        )

    @class_cached_property
    @classmethod
    def V_75MM_CANNON__SD_KFZ_171_PANTHER(cls) -> "Weapon":
        """*75MM CANNON [Sd.Kfz.171 Panther]*"""
        return cls(
            id="75MM CANNON [Sd.Kfz.171 Panther]",
            name="75mm Cannon",
            vehicle_id="Sd.Kfz.171 Panther",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.TANK_CANNON,
        )

    @class_cached_property
    @classmethod
    def V_COAXIAL_MG34__SD_KFZ_171_PANTHER(cls) -> "Weapon":
        """*COAXIAL MG34 [Sd.Kfz.171 Panther]*"""
        return cls(
            id="COAXIAL MG34 [Sd.Kfz.171 Panther]",
            name="MG34",
            vehicle_id="Sd.Kfz.171 Panther",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.TANK_COAXIAL_MG,
        )

    @class_cached_property
    @classmethod
    def V_HULL_MG34__SD_KFZ_171_PANTHER(cls) -> "Weapon":
        """*HULL MG34 [Sd.Kfz.171 Panther]*"""
        return cls(
            id="HULL MG34 [Sd.Kfz.171 Panther]",
            name="MG34",
            vehicle_id="Sd.Kfz.171 Panther",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.TANK_HULL_MG,
        )

    @class_cached_property
    @classmethod
    def V_88MM_KWK_36_L_56__SD_KFZ_181_TIGER_1(cls) -> "Weapon":
        """*88 KWK 36 L/56 [Sd.Kfz.181 Tiger 1]*"""
        return cls(
            id="88 KWK 36 L/56 [Sd.Kfz.181 Tiger 1]",
            name="88mm KwK 36 L/56",
            vehicle_id="Sd.Kfz.181 Tiger 1",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.TANK_CANNON,
        )

    @class_cached_property
    @classmethod
    def V_COAXIAL_MG34__SD_KFZ_181_TIGER_1(cls) -> "Weapon":
        """*COAXIAL MG34 [Sd.Kfz.181 Tiger 1]*"""
        return cls(
            id="COAXIAL MG34 [Sd.Kfz.181 Tiger 1]",
            name="MG34",
            vehicle_id="Sd.Kfz.181 Tiger 1",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.TANK_COAXIAL_MG,
        )

    @class_cached_property
    @classmethod
    def V_HULL_MG34__SD_KFZ_181_TIGER_1(cls) -> "Weapon":
        """*HULL MG34 [Sd.Kfz.181 Tiger 1]*"""
        return cls(
            id="HULL MG34 [Sd.Kfz.181 Tiger 1]",
            name="MG34",
            vehicle_id="Sd.Kfz.181 Tiger 1",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.TANK_HULL_MG,
        )

    @class_cached_property
    @classmethod
    def V_MG_42__SD_KFZ_251_HALF_TRACK(cls) -> "Weapon":
        """*MG 42 [Sd.Kfz 251 Half-track]*"""
        return cls(
            id="MG 42 [Sd.Kfz 251 Half-track]",
            name="MG42",
            vehicle_id="Sd.Kfz 251 Half-track",
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.MOUNTED_MG,
        )

    @class_cached_property
    @classmethod
    def V_50MM_KWK_39_1__UNKNOWN(cls) -> "Weapon":
        """*50MM KWK 39/1*"""
        return cls(
            id="50MM KWK 39/1",
            name="50mm KwK 39/1",
            vehicle_id=None,
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.TANK_CANNON,
        )

    @class_cached_property
    @classmethod
    def V_20MM_KWK_30__UNKNOWN(cls) -> "Weapon":
        """*20MM KWK 30*"""
        return cls(
            id="20MM KWK 30",
            name="20mm KwK 30",
            vehicle_id=None,
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.TANK_CANNON,
        )

    @class_cached_property
    @classmethod
    def V_88MM_KWK_36_L_56__UNKNOWN(cls) -> "Weapon":
        """*88 KWK 36 L/56*"""
        return cls(
            id="88 KWK 36 L/56",
            name="88mm KwK 36 L/56",
            vehicle_id=None,
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.TANK_CANNON,
        )

    @class_cached_property
    @classmethod
    def V_COAXIAL_MG34__UNKNOWN(cls) -> "Weapon":
        """*COAXIAL MG34*"""
        return cls(
            id="COAXIAL MG34",
            name="MG34",
            vehicle_id=None,
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.TANK_COAXIAL_MG,
        )

    @class_cached_property
    @classmethod
    def V_HULL_MG34__UNKNOWN(cls) -> "Weapon":
        """*HULL MG34*"""
        return cls(
            id="HULL MG34",
            name="MG34",
            vehicle_id=None,
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.TANK_HULL_MG,
        )

    @class_cached_property
    @classmethod
    def V_MG_42__UNKNOWN(cls) -> "Weapon":
        """*MG 42*"""
        return cls(
            id="MG 42",
            name="MG42",
            vehicle_id=None,
            factions={Faction.GER, Faction.DAK},
            type=WeaponType.MOUNTED_MG,
        )

    # --- Soviet weapons ---

    @class_cached_property
    @classmethod
    def MOSIN_NAGANT_1891(cls) -> "Weapon":
        """*MOSIN NAGANT 1891*"""
        return cls(
            id="MOSIN NAGANT 1891",
            name="Mosin-Nagant 1891",
            factions={Faction.SOV},
            type=WeaponType.BOLT_ACTION_RIFLE,
        )

    @class_cached_property
    @classmethod
    def MOSIN_NAGANT_91_30(cls) -> "Weapon":
        """*MOSIN NAGANT 91/30*"""
        return cls(
            id="MOSIN NAGANT 91/30",
            name="Mosin-Nagant 91/30",
            factions={Faction.SOV},
            type=WeaponType.BOLT_ACTION_RIFLE,
        )

    @class_cached_property
    @classmethod
    def MOSIN_NAGANT_M38(cls) -> "Weapon":
        """*MOSIN NAGANT M38*"""
        return cls(
            id="MOSIN NAGANT M38",
            name="Mosin-Nagant M38",
            factions={Faction.SOV},
            type=WeaponType.BOLT_ACTION_RIFLE,
        )

    @class_cached_property
    @classmethod
    def SVT_40(cls) -> "Weapon":
        """*SVT40*"""
        return cls(
            id="SVT40",
            name="SVT-40",
            factions={Faction.SOV},
            type=WeaponType.SEMI_AUTO_RIFLE,
        )

    @class_cached_property
    @classmethod
    def PPSH_41(cls) -> "Weapon":
        """*PPSH 41*"""
        return cls(
            id="PPSH 41",
            name="PPSh-41",
            factions={Faction.SOV},
            type=WeaponType.SUBMACHINE_GUN,
        )

    @class_cached_property
    @classmethod
    def PPSH_41_WITH_DRUM(cls) -> "Weapon":
        """*PPSH 41 W/DRUM*"""
        return cls(
            id="PPSH 41 W/DRUM",
            name="PPSh-41 with Drum",
            factions={Faction.SOV},
            type=WeaponType.SUBMACHINE_GUN,
        )

    @class_cached_property
    @classmethod
    def DP_27(cls) -> "Weapon":
        """*DP-27*"""
        return cls(
            id="DP-27",
            name="DP-27",
            factions={Faction.SOV},
            type=WeaponType.MACHINE_GUN,
        )

    @class_cached_property
    @classmethod
    def MOSIN_NAGANT_91_30_SCOPED_4X(cls) -> "Weapon":
        """*SCOPED MOSIN NAGANT 91/30*"""
        return cls(
            id="SCOPED MOSIN NAGANT 91/30",
            name="Mosin-Nagant 91/30",
            factions={Faction.SOV},
            type=WeaponType.BOLT_ACTION_RIFLE,
            magnification=4,
        )

    @class_cached_property
    @classmethod
    def SVT_40_SCOPED_4X(cls) -> "Weapon":
        """*SCOPED SVT40*"""
        return cls(
            id="SCOPED SVT40",
            name="SVT-40",
            factions={Faction.SOV},
            type=WeaponType.SEMI_AUTO_RIFLE,
            magnification=4,
        )

    @class_cached_property
    @classmethod
    def NAGANT_M1895(cls) -> "Weapon":
        """*NAGANT M1895*"""
        return cls(
            id="NAGANT M1895",
            name="Nagant M1895",
            factions={Faction.SOV},
            type=WeaponType.REVOLVER,
        )

    @class_cached_property
    @classmethod
    def TOKAREV_TT33(cls) -> "Weapon":
        """*TOKAREV TT33*"""
        return cls(
            id="TOKAREV TT33",
            name="Tokarev TT-33",
            factions={Faction.SOV},
            type=WeaponType.PISTOL,
        )

    @class_cached_property
    @classmethod
    def MPL_50_SPADE(cls) -> "Weapon":
        """*MPL-50 SPADE*"""
        return cls(
            id="MPL-50 SPADE",
            name="MPL-50 Spade",
            factions={Faction.SOV},
            type=WeaponType.MELEE,
        )

    @class_cached_property
    @classmethod
    def SATCHEL_CHARGE_SOVIET(cls) -> "Weapon":
        """*SATCHEL CHARGE*"""
        return cls(
            id="SATCHEL CHARGE",
            name="Satchel Charge",
            factions={Faction.SOV},
            type=WeaponType.SATCHEL,
        )

    @class_cached_property
    @classmethod
    def RG_42_GRENADE(cls) -> "Weapon":
        """*RG-42 GRENADE*"""
        return cls(
            id="RG-42 GRENADE",
            name="RG-42 Grenade",
            factions={Faction.SOV},
            type=WeaponType.GRENADE,
        )

    @class_cached_property
    @classmethod
    def MOLOTOV(cls) -> "Weapon":
        """*MOLOTOV*"""
        return cls(
            id="MOLOTOV",
            name="Molotov",
            factions={Faction.SOV},
            type=WeaponType.GRENADE,
        )

    @class_cached_property
    @classmethod
    def PTRS_41(cls) -> "Weapon":
        """*PTRS-41*"""
        return cls(
            id="PTRS-41",
            name="PTRS-41",
            factions={Faction.SOV},
            type=WeaponType.ANTI_MATERIEL_RIFLE,
        )

    @class_cached_property
    @classmethod
    def POMZ_AP_MINE(cls) -> "Weapon":
        """*POMZ AP MINE*"""
        return cls(
            id="POMZ AP MINE",
            name="POMZ AP Mine",
            factions={Faction.SOV},
            type=WeaponType.AP_MINE,
        )

    @class_cached_property
    @classmethod
    def TM_35_AT_MINE(cls) -> "Weapon":
        """*TM-35 AT MINE*"""
        return cls(
            id="TM-35 AT MINE",
            name="TM-35 AT Mine",
            factions={Faction.SOV},
            type=WeaponType.AT_MINE,
        )

    # FLARE GUN

    @class_cached_property
    @classmethod
    def V_57MM_CANNON__ZIS_2(cls) -> "Weapon":
        """*57MM CANNON [ZiS-2]*"""
        return cls(
            id="57MM CANNON [ZiS-2]",
            name="57mm Cannon",
            vehicle_id="ZiS-2",
            factions={Faction.SOV},
            type=WeaponType.AT_GUN,
        )

    @class_cached_property
    @classmethod
    def V_122MM_HOWITZER__M1938_M_30(cls) -> "Weapon":
        """*122MM HOWITZER [M1938 (M-30)]*"""
        return cls(
            id="122MM HOWITZER [M1938 (M-30)]",
            name="122mm Howitzer",
            vehicle_id="M1938 (M-30)",
            factions={Faction.SOV},
            type=WeaponType.ARTILLERY,
        )

    @class_cached_property
    @classmethod
    def V_ROADKILL__BA_10(cls) -> "Weapon":
        """*BA-10*"""
        return cls(
            id="BA-10",
            name="Roadkill",
            vehicle_id="BA-10",
            factions={Faction.SOV},
            type=WeaponType.ROADKILL,
        )

    @class_cached_property
    @classmethod
    def V_ROADKILL__T70(cls) -> "Weapon":
        """*T70*"""
        return cls(
            id="T70",
            name="Roadkill",
            vehicle_id="T70",
            factions={Faction.SOV},
            type=WeaponType.ROADKILL,
        )

    @class_cached_property
    @classmethod
    def V_ROADKILL__T34_76(cls) -> "Weapon":
        """*T34/76*"""
        return cls(
            id="T34/76",
            name="Roadkill",
            vehicle_id="T34/76",
            factions={Faction.SOV},
            type=WeaponType.ROADKILL,
        )

    @class_cached_property
    @classmethod
    def V_ROADKILL__IS_1(cls) -> "Weapon":
        """*IS-1*"""
        return cls(
            id="IS-1",
            name="Roadkill",
            vehicle_id="IS-1",
            factions={Faction.SOV},
            type=WeaponType.ROADKILL,
        )

    @class_cached_property
    @classmethod
    def V_ROADKILL__ZIS_5_SUPPLY(cls) -> "Weapon":
        """*ZIS-5 (Supply)*"""
        return cls(
            id="ZIS-5 (Supply)",
            name="Roadkill",
            vehicle_id="ZIS-5 (Supply)",
            factions={Faction.SOV},
            type=WeaponType.ROADKILL,
        )

    @class_cached_property
    @classmethod
    def V_ROADKILL__ZIS_5_TRANSPORT(cls) -> "Weapon":
        """*ZIS-5 (Transport)*"""
        return cls(
            id="ZIS-5 (Transport)",
            name="Roadkill",
            vehicle_id="ZIS-5 (Transport)",
            factions={Faction.SOV},
            type=WeaponType.ROADKILL,
        )

    # M3 Half-track,

    @class_cached_property
    @classmethod
    def V_ROADKILL__GAZ_67(cls) -> "Weapon":
        """*GAZ-67*"""
        return cls(
            id="GAZ-67",
            name="Roadkill",
            vehicle_id="GAZ-67",
            factions={Faction.SOV},
            type=WeaponType.ROADKILL,
        )

    @class_cached_property
    @classmethod
    def V_19_K_45MM__BA_10(cls) -> "Weapon":
        """*19-K 45MM [BA-10]*"""
        return cls(
            id="19-K 45MM [BA-10]",
            name="45mm M1932",
            vehicle_id="BA-10",
            factions={Faction.SOV},
            type=WeaponType.TANK_CANNON,
        )

    @class_cached_property
    @classmethod
    def V_COAXIAL_DT__BA_10(cls) -> "Weapon":
        """*COAXIAL DT [BA-10]*"""
        return cls(
            id="COAXIAL DT [BA-10]",
            name="DT",
            vehicle_id="BA-10",
            factions={Faction.SOV},
            type=WeaponType.TANK_COAXIAL_MG,
        )

    @class_cached_property
    @classmethod
    def V_45MM_M1937__T70(cls) -> "Weapon":
        """*45MM M1937 [T70]*"""
        return cls(
            id="45MM M1937 [T70]",
            name="45mm M1937",
            vehicle_id="T70",
            factions={Faction.SOV},
            type=WeaponType.TANK_CANNON,
        )

    @class_cached_property
    @classmethod
    def V_COAXIAL_DT__T70(cls) -> "Weapon":
        """*COAXIAL DT [T70]*"""
        return cls(
            id="COAXIAL DT [T70]",
            name="DT",
            vehicle_id="T70",
            factions={Faction.SOV},
            type=WeaponType.TANK_COAXIAL_MG,
        )

    @class_cached_property
    @classmethod
    def V_76MM_ZIS_5__T34_76(cls) -> "Weapon":
        """*76MM ZiS-5 [T34/76]*"""
        return cls(
            id="76MM ZiS-5 [T34/76]",
            name="76mm M1940",
            vehicle_id="T34/76",
            factions={Faction.SOV},
            type=WeaponType.TANK_CANNON,
        )

    @class_cached_property
    @classmethod
    def V_COAXIAL_DT__T34_76(cls) -> "Weapon":
        """*COAXIAL DT [T34/76]*"""
        return cls(
            id="COAXIAL DT [T34/76]",
            name="DT",
            vehicle_id="T34/76",
            factions={Faction.SOV},
            type=WeaponType.TANK_COAXIAL_MG,
        )

    @class_cached_property
    @classmethod
    def V_HULL_DT__T34_76(cls) -> "Weapon":
        """*HULL DT [T34/76]*"""
        return cls(
            id="HULL DT [T34/76]",
            name="DT",
            vehicle_id="T34/76",
            factions={Faction.SOV},
            type=WeaponType.TANK_HULL_MG,
        )

    @class_cached_property
    @classmethod
    def V_D_5T_85MM__IS_1(cls) -> "Weapon":
        """*D-5T 85MM [IS-1]*"""
        return cls(
            id="D-5T 85MM [IS-1]",
            name="D-5T 85mm",
            vehicle_id="IS-1",
            factions={Faction.SOV},
            type=WeaponType.TANK_CANNON,
        )

    @class_cached_property
    @classmethod
    def V_COAXIAL_DT__IS_1(cls) -> "Weapon":
        """*COAXIAL DT [IS-1]*"""
        return cls(
            id="COAXIAL DT [IS-1]",
            name="DT",
            vehicle_id="IS-1",
            factions={Faction.SOV},
            type=WeaponType.TANK_COAXIAL_MG,
        )

    @class_cached_property
    @classmethod
    def V_HULL_DT__IS_1(cls) -> "Weapon":
        """*HULL DT [IS-1]*"""
        return cls(
            id="HULL DT [IS-1]",
            name="DT",
            vehicle_id="IS-1",
            factions={Faction.SOV},
            type=WeaponType.TANK_HULL_MG,
        )

    # M2 Browning [M3 Half-track]

    @class_cached_property
    @classmethod
    def V_19_K_45MM__UNKNOWN(cls) -> "Weapon":
        """*19-K 45MM*"""
        return cls(
            id="19-K 45MM",
            name="45mm M1932",
            vehicle_id=None,
            factions={Faction.SOV},
            type=WeaponType.TANK_CANNON,
        )

    @class_cached_property
    @classmethod
    def V_45MM_M1937__UNKNOWN(cls) -> "Weapon":
        """*45MM M1937*"""
        return cls(
            id="45MM M1937",
            name="45mm M1937",
            vehicle_id=None,
            factions={Faction.SOV},
            type=WeaponType.TANK_CANNON,
        )

    @class_cached_property
    @classmethod
    def V_76MM_ZIS_5__UNKNOWN(cls) -> "Weapon":
        """*76MM ZiS-5*"""
        return cls(
            id="76MM ZiS-5",
            name="76mm M1940",
            vehicle_id=None,
            factions={Faction.SOV},
            type=WeaponType.TANK_CANNON,
        )

    @class_cached_property
    @classmethod
    def V_D_5T_85MM__UNKNOWN(cls) -> "Weapon":
        """*D-5T 85MM*"""
        return cls(
            id="D-5T 85MM",
            name="D-5T 85mm",
            vehicle_id=None,
            factions={Faction.SOV},
            type=WeaponType.TANK_CANNON,
        )

    @class_cached_property
    @classmethod
    def V_COAXIAL_DT__UNKNOWN(cls) -> "Weapon":
        """*COAXIAL DT*"""
        return cls(
            id="COAXIAL DT",
            name="COAXIAL DT",
            vehicle_id=None,
            factions={Faction.SOV},
            type=WeaponType.TANK_COAXIAL_MG,
        )

    @class_cached_property
    @classmethod
    def V_HULL_DT__UNKNOWN(cls) -> "Weapon":
        """*HULL DT*"""
        return cls(
            id="HULL DT",
            name="HULL DT",
            vehicle_id=None,
            factions={Faction.SOV},
            type=WeaponType.TANK_HULL_MG,
        )

    # --- British weapons ---

    @class_cached_property
    @classmethod
    def SMLE_NO_1_MK_III(cls) -> "Weapon":
        """*SMLE No.1 Mk III*"""
        return cls(
            id="SMLE No.1 Mk III",
            name="SMLE Mk III",
            factions={Faction.CW, Faction.B8A},
            type=WeaponType.BOLT_ACTION_RIFLE,
        )

    @class_cached_property
    @classmethod
    def RIFLE_NO_4_MK_I(cls) -> "Weapon":
        """*Rifle No.4 Mk I*"""
        return cls(
            id="Rifle No.4 Mk I",
            name="No.4 Rifle Mk I",
            factions={Faction.CW, Faction.B8A},
            type=WeaponType.BOLT_ACTION_RIFLE,
        )

    @class_cached_property
    @classmethod
    def STEN_GUN(cls) -> "Weapon":
        """*Sten Gun*"""
        return cls(
            id="Sten Gun",
            name="Sten Mk II",
            factions={Faction.B8A},
            type=WeaponType.SUBMACHINE_GUN,
        )

    @class_cached_property
    @classmethod
    def STEN_GUN_MK_II(cls) -> "Weapon":
        """*Sten Gun Mk.II*"""
        return cls(
            id="Sten Gun Mk.II",
            name="Sten Mk II",
            factions={Faction.CW},
            type=WeaponType.SUBMACHINE_GUN,
        )

    @class_cached_property
    @classmethod
    def STEN_GUN_MK_V(cls) -> "Weapon":
        """*Sten Gun Mk.V*"""
        return cls(
            id="Sten Gun Mk.V",
            name="Sten Mk V",
            factions={Faction.CW},
            type=WeaponType.SUBMACHINE_GUN,
        )

    @class_cached_property
    @classmethod
    def M1928A1_THOMPSON(cls) -> "Weapon":
        """*M1928A1 THOMPSON*"""
        return cls(
            id="M1928A1 THOMPSON",
            name="M1928A1 Thompson",
            factions={Faction.CW, Faction.B8A},
            type=WeaponType.SUBMACHINE_GUN,
        )

    @class_cached_property
    @classmethod
    def BREN_GUN(cls) -> "Weapon":
        """*Bren Gun*"""
        return cls(
            id="Bren Gun",
            name="Bren Gun",
            factions={Faction.CW, Faction.B8A},
            type=WeaponType.ASSAULT_RIFLE,
        )

    @class_cached_property
    @classmethod
    def LEWIS_GUN(cls) -> "Weapon":
        """*Lewis Gun*"""
        return cls(
            id="Lewis Gun",
            name="Lewis Gun",
            factions={Faction.CW, Faction.B8A},
            type=WeaponType.MACHINE_GUN,
        )

    @class_cached_property
    @classmethod
    def LIFEBUOY_FLAMETHROWER(cls) -> "Weapon":
        """*FLAMETHROWER*"""
        return cls(
            id="FLAMETHROWER",
            name="Lifebuoy Flamethrower",
            factions={Faction.CW, Faction.B8A},
            type=WeaponType.FLAMETHROWER,
        )

    @class_cached_property
    @classmethod
    def RIFLE_NO_4_MK_I_SCOPED_8X(cls) -> "Weapon":
        """*Rifle No.4 Mk I Sniper*"""
        return cls(
            id="Rifle No.4 Mk I Sniper",
            name="No.4 Rifle Mk I",
            factions={Faction.CW, Faction.B8A},
            type=WeaponType.BOLT_ACTION_RIFLE,
            magnification=8,
        )

    @class_cached_property
    @classmethod
    def WEBLEY_MK_VI(cls) -> "Weapon":
        """*Webley MK VI*"""
        return cls(
            id="Webley MK VI",
            name="Webley Mk IV",
            factions={Faction.CW, Faction.B8A},
            type=WeaponType.REVOLVER,
        )

    @class_cached_property
    @classmethod
    def FAIRBAIRN_SYKES(cls) -> "Weapon":
        """*Fairbairn–Sykes*"""  # noqa: RUF002
        return cls(
            id="Fairbairn–Sykes",  # noqa: RUF001
            name="Fairbairn-Sykes",
            factions={Faction.CW, Faction.B8A},
            type=WeaponType.MELEE,
        )

    @class_cached_property
    @classmethod
    def SATCHEL_CHARGE_COMMONWEALTH(cls) -> "Weapon":
        """*Satchel*"""
        return cls(
            id="Satchel",
            name="Satchel Charge",
            factions={Faction.CW, Faction.B8A},
            type=WeaponType.SATCHEL,
        )

    @class_cached_property
    @classmethod
    def MILLS_BOMB(cls) -> "Weapon":
        """*Mills Bomb*"""
        return cls(
            id="Mills Bomb",
            name="Mills Bomb",
            factions={Faction.CW, Faction.B8A},
            type=WeaponType.GRENADE,
        )

    @class_cached_property
    @classmethod
    def GAMMON_BOMB(cls) -> "Weapon":
        """*No.82 Grenade*"""
        return cls(
            id="No.82 Grenade",
            name="Gammon Bomb",
            factions={Faction.CW},
            type=WeaponType.GRENADE,
        )

    @class_cached_property
    @classmethod
    def PIAT(cls) -> "Weapon":
        """*PIAT*"""
        return cls(
            id="PIAT",
            name="PIAT",
            factions={Faction.CW, Faction.B8A},
            type=WeaponType.ROCKET_LAUNCHER,
        )

    @class_cached_property
    @classmethod
    def BOYS_ANTI_TANK_RIFLE(cls) -> "Weapon":
        """*Boys Anti-tank Rifle*"""
        return cls(
            id="Boys Anti-tank Rifle",
            name="Boys AT Rifle",
            factions={Faction.CW, Faction.B8A},
            type=WeaponType.ANTI_MATERIEL_RIFLE,
        )

    @class_cached_property
    @classmethod
    def AP_SHRAPNEL_MINE_MK_II(cls) -> "Weapon":
        """*A.P. Shrapnel Mine Mk II*"""
        return cls(
            id="A.P. Shrapnel Mine Mk II",
            name="AP Shrapnel Mine Mk II",
            factions={Faction.CW, Faction.B8A},
            type=WeaponType.AP_MINE,
        )

    @class_cached_property
    @classmethod
    def AT_MINE_GS_MK_V(cls) -> "Weapon":
        """*A.T. Mine G.S. Mk V*"""
        return cls(
            id="A.T. Mine G.S. Mk V",
            name="AT Mine G.S. Mk V",
            factions={Faction.CW, Faction.B8A},
            type=WeaponType.AT_MINE,
        )

    @class_cached_property
    @classmethod
    def NO_2_MK_V_FLARE_GUN(cls) -> "Weapon":
        """*No.2 Mk 5 Flare Pistol*"""
        return cls(
            id="No.2 Mk 5 Flare Pistol",
            name="No.2 Mk V Flare Gun",
            factions={Faction.CW, Faction.B8A},
            type=WeaponType.FLARE_GUN,
        )

    @class_cached_property
    @classmethod
    def V_QF_6_POUNDER__QF_6_POUNDER(cls) -> "Weapon":
        """*QF 6-POUNDER [QF 6-Pounder]*"""
        return cls(
            id="QF 6-POUNDER [QF 6-Pounder]",
            name="57mm Cannon",
            vehicle_id="QF 6-Pounder",
            factions={Faction.CW, Faction.B8A},
            type=WeaponType.AT_GUN,
        )

    @class_cached_property
    @classmethod
    def V_QF_25_POUNDER__QF_25_POUNDER(cls) -> "Weapon":
        """*QF 25-POUNDER [QF 25-Pounder]*"""
        return cls(
            id="QF 25-POUNDER [QF 25-Pounder]",
            name="88mm Howitzer",
            vehicle_id="QF 25-Pounder",
            factions={Faction.CW, Faction.B8A},
            type=WeaponType.ARTILLERY,
        )

    @class_cached_property
    @classmethod
    def V_ROADKILL__DAIMLER(cls) -> "Weapon":
        """*Daimler*"""
        return cls(
            id="Daimler",
            name="Roadkill",
            vehicle_id="Daimler",
            factions={Faction.CW, Faction.B8A},
            type=WeaponType.ROADKILL,
        )

    @class_cached_property
    @classmethod
    def V_ROADKILL__TETRARCH(cls) -> "Weapon":
        """*Tetrarch*"""
        return cls(
            id="Tetrarch",
            name="Roadkill",
            vehicle_id="Tetrarch",
            factions={Faction.CW},
            type=WeaponType.ROADKILL,
        )

    @class_cached_property
    @classmethod
    def V_ROADKILL__M3_STUART_HONEY(cls) -> "Weapon":
        """*M3 Stuart Honey*"""
        return cls(
            id="M3 Stuart Honey",
            name="Roadkill",
            vehicle_id="M3 Stuart Honey",
            factions={Faction.B8A},
            type=WeaponType.ROADKILL,
        )

    @class_cached_property
    @classmethod
    def V_ROADKILL__CROMWELL(cls) -> "Weapon":
        """*Cromwell*"""
        return cls(
            id="Cromwell",
            name="Roadkill",
            vehicle_id="Cromwell",
            factions={Faction.CW},
            type=WeaponType.ROADKILL,
        )

    @class_cached_property
    @classmethod
    def V_ROADKILL__CRUSADER_MK_III(cls) -> "Weapon":
        """*Crusader Mk.III*"""
        return cls(
            id="Crusader Mk.III",
            name="Roadkill",
            vehicle_id="Crusader Mk.III",
            factions={Faction.B8A},
            type=WeaponType.ROADKILL,
        )

    @class_cached_property
    @classmethod
    def V_ROADKILL__FIREFLY(cls) -> "Weapon":
        """*Firefly*"""
        return cls(
            id="Firefly",
            name="Roadkill",
            vehicle_id="Firefly",
            factions={Faction.CW},
            type=WeaponType.ROADKILL,
        )

    @class_cached_property
    @classmethod
    def V_ROADKILL__CHURCHILL_MK_III(cls) -> "Weapon":
        """*Churchill Mk.III*"""
        return cls(
            id="Churchill Mk.III",
            name="Roadkill",
            vehicle_id="Churchill Mk.III",
            factions={Faction.B8A},
            type=WeaponType.ROADKILL,
        )

    @class_cached_property
    @classmethod
    def V_ROADKILL__CHURCHILL_MK_VII(cls) -> "Weapon":
        """*Churchill Mk.VII*"""
        return cls(
            id="Churchill Mk.VII",
            name="Roadkill",
            vehicle_id="Churchill Mk.VII",
            factions={Faction.CW},
            type=WeaponType.ROADKILL,
        )

    @class_cached_property
    @classmethod
    def V_ROADKILL__BEDFORD_OYD_SUPPLY(cls) -> "Weapon":
        """*Bedford OYD (Supply)*"""
        return cls(
            id="Bedford OYD (Supply)",
            name="Roadkill",
            vehicle_id="Bedford OYD (Supply)",
            factions={Faction.CW, Faction.B8A},
            type=WeaponType.ROADKILL,
        )

    @class_cached_property
    @classmethod
    def V_ROADKILL__BEDFORD_OYD_TRANSPORT(cls) -> "Weapon":
        """*Bedford OYD (Transport)*"""
        return cls(
            id="Bedford OYD (Transport)",
            name="Roadkill",
            vehicle_id="Bedford OYD (Transport)",
            factions={Faction.CW, Faction.B8A},
            type=WeaponType.ROADKILL,
        )

    # M3 Half-track,

    # Jeep Willys

    @class_cached_property
    @classmethod
    def V_QF_2_POUNDER__DAIMLER(cls) -> "Weapon":
        """*QF 2-POUNDER [Daimler]*"""
        return cls(
            id="QF 2-POUNDER [Daimler]",
            name="QF 2-Pounder",
            vehicle_id="Daimler",
            factions={Faction.CW, Faction.B8A},
            type=WeaponType.TANK_CANNON,
        )

    @class_cached_property
    @classmethod
    def V_COAXIAL_BESA__DAIMLER(cls) -> "Weapon":
        """*COAXIAL BESA [Daimler]*"""
        return cls(
            id="COAXIAL BESA [Daimler]",
            name="BESA",
            vehicle_id="Daimler",
            factions={Faction.CW, Faction.B8A},
            type=WeaponType.TANK_COAXIAL_MG,
        )

    @class_cached_property
    @classmethod
    def V_QF_2_POUNDER__TETRARCH(cls) -> "Weapon":
        """*QF 2-POUNDER [Tetrarch]*"""
        return cls(
            id="QF 2-POUNDER [Tetrarch]",
            name="QF 2-Pounder",
            vehicle_id="Tetrarch",
            factions={Faction.CW},
            type=WeaponType.TANK_CANNON,
        )

    @class_cached_property
    @classmethod
    def V_COAXIAL_BESA__TETRARCH(cls) -> "Weapon":
        """*COAXIAL BESA [Tetrarch]*"""
        return cls(
            id="COAXIAL BESA [Tetrarch]",
            name="BESA",
            vehicle_id="Tetrarch",
            factions={Faction.CW},
            type=WeaponType.TANK_COAXIAL_MG,
        )

    @class_cached_property
    @classmethod
    def V_37MM_CANNON__M3_STUART_HONEY(cls) -> "Weapon":
        """*37MM CANNON [M3 Stuart Honey]*"""
        return cls(
            id="37MM CANNON [M3 Stuart Honey]",
            name="37mm Cannon",
            vehicle_id="M3 Stuart Honey",
            factions={Faction.B8A},
            type=WeaponType.TANK_CANNON,
        )

    @class_cached_property
    @classmethod
    def V_COAXIAL_M1919__M3_STUART_HONEY(cls) -> "Weapon":
        """*COAXIAL M1919 [M3 Stuart Honey]*"""
        return cls(
            id="COAXIAL M1919 [M3 Stuart Honey]",
            name="M1919 Browning",
            vehicle_id="M3 Stuart Honey",
            factions={Faction.B8A},
            type=WeaponType.TANK_COAXIAL_MG,
        )

    @class_cached_property
    @classmethod
    def V_HULL_M1919__M3_STUART_HONEY(cls) -> "Weapon":
        """*HULL M1919 [M3 Stuart Honey]*"""
        return cls(
            id="HULL M1919 [M3 Stuart Honey]",
            name="M1919 Browning",
            vehicle_id="M3 Stuart Honey",
            factions={Faction.B8A},
            type=WeaponType.TANK_HULL_MG,
        )

    @class_cached_property
    @classmethod
    def V_OQF_75MM__CROMWELL(cls) -> "Weapon":
        """*OQF 75MM [Cromwell]*"""
        return cls(
            id="OQF 75MM [Cromwell]",
            name="QF 75mm",
            vehicle_id="Cromwell",
            factions={Faction.CW},
            type=WeaponType.TANK_CANNON,
        )

    @class_cached_property
    @classmethod
    def V_COAXIAL_BESA__CROMWELL(cls) -> "Weapon":
        """*COAXIAL BESA [Cromwell]*"""
        return cls(
            id="COAXIAL BESA [Cromwell]",
            name="BESA",
            vehicle_id="Cromwell",
            factions={Faction.CW},
            type=WeaponType.TANK_COAXIAL_MG,
        )

    @class_cached_property
    @classmethod
    def V_HULL_BESA__CROMWELL(cls) -> "Weapon":
        """*HULL BESA [Cromwell]*"""
        return cls(
            id="HULL BESA [Cromwell]",
            name="BESA",
            vehicle_id="Cromwell",
            factions={Faction.CW},
            type=WeaponType.TANK_HULL_MG,
        )

    @class_cached_property
    @classmethod
    def V_OQF_57MM__CRUSADER_MK_III(cls) -> "Weapon":
        """*OQF 57MM [Crusader Mk.III]*"""
        return cls(
            id="OQF 57MM [Crusader Mk.III]",
            name="QF 57mm",
            vehicle_id="Crusader Mk.III",
            factions={Faction.B8A},
            type=WeaponType.TANK_CANNON,
        )

    @class_cached_property
    @classmethod
    def V_COAXIAL_BESA__CRUSADER_MK_III(cls) -> "Weapon":
        """*COAXIAL BESA [Crusader Mk.III]*"""
        return cls(
            id="COAXIAL BESA [Crusader Mk.III]",
            name="BESA",
            vehicle_id="Crusader Mk.III",
            factions={Faction.B8A},
            type=WeaponType.TANK_COAXIAL_MG,
        )

    @class_cached_property
    @classmethod
    def V_QF_17_POUNDER__FIREFLY(cls) -> "Weapon":
        """*QF 17-POUNDER [Firefly]*"""
        return cls(
            id="QF 17-POUNDER [Firefly]",
            name="QF 17-Pounder",
            vehicle_id="Firefly",
            factions={Faction.CW},
            type=WeaponType.TANK_CANNON,
        )

    @class_cached_property
    @classmethod
    def V_COAXIAL_M1919__FIREFLY(cls) -> "Weapon":
        """*COAXIAL M1919 [Firefly]*"""
        return cls(
            id="COAXIAL M1919 [Firefly]",
            name="M1919 Browning",
            vehicle_id="Firefly",
            factions={Faction.CW},
            type=WeaponType.TANK_COAXIAL_MG,
        )

    @class_cached_property
    @classmethod
    def V_OQF_57MM__CHURCHILL_MK_III(cls) -> "Weapon":
        """*OQF 57MM [Churchill Mk.III]*"""
        return cls(
            id="OQF 57MM [Churchill Mk.III]",
            name="QF 57mm",
            vehicle_id="Churchill Mk.III",
            factions={Faction.B8A},
            type=WeaponType.TANK_CANNON,
        )

    @class_cached_property
    @classmethod
    def V_COAXIAL_BESA_7_92MM__CHURCHILL_MK_III(cls) -> "Weapon":
        """*COAXIAL BESA 7.92mm [Churchill Mk.III]*"""
        return cls(
            id="COAXIAL BESA 7.92mm [Churchill Mk.III]",
            name="BESA",
            vehicle_id="Churchill Mk.III",
            factions={Faction.B8A},
            type=WeaponType.TANK_COAXIAL_MG,
        )

    @class_cached_property
    @classmethod
    def V_HULL_BESA_7_92MM__CHURCHILL_MK_III(cls) -> "Weapon":
        """*HULL BESA 7.92mm [Churchill Mk.III]*"""
        return cls(
            id="HULL BESA 7.92mm [Churchill Mk.III]",
            name="BESA",
            vehicle_id="Churchill Mk.III",
            factions={Faction.B8A},
            type=WeaponType.TANK_HULL_MG,
        )

    @class_cached_property
    @classmethod
    def V_OQF_75MM__CHURCHILL_MK_VII(cls) -> "Weapon":
        """*OQF 75MM [Churchill Mk.VII]*"""
        return cls(
            id="OQF 75MM [Churchill Mk.VII]",
            name="QF 75mm",
            vehicle_id="Churchill Mk.VII",
            factions={Faction.CW},
            type=WeaponType.TANK_CANNON,
        )

    @class_cached_property
    @classmethod
    def V_COAXIAL_BESA_7_92MM__CHURCHILL_MK_VII(cls) -> "Weapon":
        """*COAXIAL BESA 7.92mm [Churchill Mk.VII]*"""
        return cls(
            id="COAXIAL BESA 7.92mm [Churchill Mk.VII]",
            name="BESA",
            vehicle_id="Churchill Mk.VII",
            factions={Faction.CW},
            type=WeaponType.TANK_COAXIAL_MG,
        )

    @class_cached_property
    @classmethod
    def V_HULL_BESA_7_92MM__CHURCHILL_MK_VII(cls) -> "Weapon":
        """*HULL BESA 7.92mm [Churchill Mk.VII]*"""
        return cls(
            id="HULL BESA 7.92mm [Churchill Mk.VII]",
            name="BESA",
            vehicle_id="Churchill Mk.VII",
            factions={Faction.CW},
            type=WeaponType.TANK_HULL_MG,
        )

    # M2 Browning [M3 Half-track]

    @class_cached_property
    @classmethod
    def V_QF_2_POUNDER__UNKNOWN(cls) -> "Weapon":
        """*QF 2-POUNDER*"""
        return cls(
            id="QF 2-POUNDER",
            name="QF 2-Pounder",
            vehicle_id=None,
            factions={Faction.CW, Faction.B8A},
            type=WeaponType.TANK_CANNON,
        )

    @class_cached_property
    @classmethod
    def V_OQF_75MM__UNKNOWN(cls) -> "Weapon":
        """*OQF 75MM*"""
        return cls(
            id="OQF 75MM",
            name="QF 75mm",
            vehicle_id=None,
            factions={Faction.CW},
            type=WeaponType.TANK_CANNON,
        )

    @class_cached_property
    @classmethod
    def V_OQF_57MM__UNKNOWN(cls) -> "Weapon":
        """*OQF 57MM*"""
        return cls(
            id="OQF 57MM",
            name="QF 57mm",
            vehicle_id=None,
            factions={Faction.B8A},
            type=WeaponType.TANK_CANNON,
        )

    @class_cached_property
    @classmethod
    def V_QF_17_POUNDER__UNKNOWN(cls) -> "Weapon":
        """*QF 17-POUNDER*"""
        return cls(
            id="QF 17-POUNDER",
            name="QF 17-Pounder",
            vehicle_id=None,
            factions={Faction.CW},
            type=WeaponType.TANK_CANNON,
        )

    @class_cached_property
    @classmethod
    def V_COAXIAL_BESA__UNKNOWN(cls) -> "Weapon":
        """*COAXIAL BESA*"""
        return cls(
            id="COAXIAL BESA",
            name="BESA",
            vehicle_id=None,
            factions={Faction.CW, Faction.B8A},
            type=WeaponType.TANK_COAXIAL_MG,
        )

    @class_cached_property
    @classmethod
    def V_COAXIAL_BESA_7_92MM__UNKNOWN(cls) -> "Weapon":
        """*COAXIAL BESA 7.92mm*"""
        return cls(
            id="COAXIAL BESA 7.92mm",
            name="BESA",
            vehicle_id=None,
            factions={Faction.CW, Faction.B8A},
            type=WeaponType.TANK_COAXIAL_MG,
        )

    @class_cached_property
    @classmethod
    def V_HULL_BESA__UNKNOWN(cls) -> "Weapon":
        """*HULL BESA*"""
        return cls(
            id="HULL BESA",
            name="BESA",
            vehicle_id=None,
            factions={Faction.CW, Faction.B8A},
            type=WeaponType.TANK_HULL_MG,
        )

    @class_cached_property
    @classmethod
    def V_HULL_BESA_7_92MM__UNKNOWN(cls) -> "Weapon":
        """*HULL BESA 7.92mm*"""
        return cls(
            id="HULL BESA 7.92mm",
            name="7.92mm",
            vehicle_id=None,
            factions={Faction.CW, Faction.B8A},
            type=WeaponType.TANK_HULL_MG,
        )

    # --- Miscellaneous weapons ---

    @class_cached_property
    @classmethod
    def UNKNOWN(cls) -> "Weapon":
        """*UNKNOWN*"""
        return cls(
            id="UNKNOWN",
            name="Unknown",
            factions=set(Faction.all()),
            type=WeaponType.UNKNOWN,
        )

    @class_cached_property
    @classmethod
    def BOMBING_RUN(cls) -> "Weapon":
        """*BOMBING RUN*"""
        return cls(
            id="BOMBING RUN",
            name="Bombing Run",
            factions=set(Faction.all()) - {Faction.SOV},
            type=WeaponType.COMMANDER_ABILITY,
        )

    @class_cached_property
    @classmethod
    def STRAFING_RUN(cls) -> "Weapon":
        """*STRAFING RUN*"""
        return cls(
            id="STRAFING RUN",
            name="Strafing Run",
            factions=set(Faction.all()),
            type=WeaponType.COMMANDER_ABILITY,
        )

    @class_cached_property
    @classmethod
    def PRECISION_STRIKE(cls) -> "Weapon":
        """*PRECISION STRIKE*"""
        return cls(
            id="PRECISION STRIKE",
            name="Precision Strike",
            factions=set(Faction.all()),
            type=WeaponType.COMMANDER_ABILITY,
        )

    @class_cached_property
    @classmethod
    def ARTILLERY_STRIKE(cls) -> "Weapon":
        """*Unknown*"""
        return cls(
            id="Unknown",
            name="Artillery Strike",
            factions=set(Faction.all()),
            type=WeaponType.COMMANDER_ABILITY,
        )

    @class_cached_property
    @classmethod
    def KATYUSHA_BARRAGE(cls) -> "Weapon":
        """*Unknown*"""
        return cls.ARTILLERY_STRIKE

    @class_cached_property
    @classmethod
    def FIRE_SPOT(cls) -> "Weapon":
        """*FireSpot*"""
        return cls(
            id="FireSpot",
            name="Fire",
            factions=set(Faction.all()),
            type=WeaponType.UNKNOWN,
        )
